//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: ArrayValue.java,v 1.8 2001/03/12 23:56:46 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ArrayValue.java,v $
//  Revision 1.8  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.7  2001/02/01 22:34:29  cvs
//  Added some accessor methods to the classes which are used in DVTG
//  --srinivas
//
//  Revision 1.6  2000/11/13 21:49:10  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.5  2000/10/30 22:34:30  cvs
//  Added secondary constructors to some of the classes and some
//  accessor functions are also added 
//  --srinivas
//
//  Revision 1.4  2000/09/18 20:41:02  cvs
//  Added correct tabbing to files. Also added CVS log info to new files
//  added by Steve Parker (Averstar).
//  -- Cindy
//
//  Revision 1.3  2000/08/30 20:39:10  cvs
//  Changed Expression class to inherit from Value and amended all
//  other classes affected by this change. -David 
//
//  Revision 1.2  2000/08/15 20:26:53  ckong
//  *** empty log message ***
//
//  Revision 1.1  2000/08/11 16:28:24  cvs
//  Replacing Array.java, Set.java, and Sequence.java with files named
//  xxxValue.java. All three are a bit more fleshed out as well, with
//  accessors. Purpose of replacement: To distinguish these classes
//  from those of same name to appear (soon) as subclasses of
//  Type. -David 
//
//  Revision 1.2  2000/08/10 01:04:11  cvs
//  All object in ROM has a parent, therefore implemented the constructors
//  to take in a BaseObject parent.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:06  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.1  2000/08/09 15:44:14  cvs
//  Just adding some class skeltons for various subclasses of Expression
//
//
//=============================================================================
package ROM;
import java.io.*;
import java.util.*;
/** This represents the <B>Array</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */

public class ArrayValue 
extends Expression {

	Type type=null;
	ExpressionList elements=null;

	//changed the type of param from Item to BaseObject
	/** The primary constructor
	 * @param parent The BaseObject parent of an object of this class
	 * @param typeIn The type of the elements in the array
	 * @param elementsIn The list of the elements of the array
	 * @return Not applicable
	 */
	public ArrayValue(BaseObject parent, Type typeIn, 
					  ExpressionList elementsIn) {
		super(parent);
		type = typeIn;
		elements = elementsIn;
	}
	
	/** The secondary constructor
	 * @param parent The BaseObject parent of an object of this class
	 * @return Not applicable
	 */
	public ArrayValue(BaseObject parent) {
		super(parent);
	}

	/** To retrieve element type
	 * @return Type representing the type of elements in the array
	 */
        public Type getType(){
	        return type;
	}

	/** To return the array
	 * @return ItemList The list representing the array
	 */
        public ExpressionList getElements(){
	    return elements;
	}


	/**To set the type
	 * @param Type
	 * @return void
	 */
	public void setType(Type typeIn){
		type = typeIn;
	}

	/**To set the elements
	 * @param ItemList
	 * @return void
	 */
	public void setElements(ExpressionList itemIn){
		elements = itemIn;
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		return null;
	}

	/** For cases when value references an item.
	 * Irrelevant here.
	 */
	public LabelledItem labelLookup(Label label2){
		return null;
	}
	
	public void publishROM(PrintWriter pw){
		pw.print("[");
		ListIterator exprIter = getElements().listIterator();
		if (exprIter != null){
			while (exprIter.hasNext()){
				Expression expr = (Expression)exprIter.next();
				if (expr != null)
					expr.publishROM(pw);
			}
		}
		pw.print("]");
	}
	
}

		


