//=============================================================================
//	authors: Cindy Kong
//
//  $Id: BitType.java,v 1.2 2001/05/25 21:05:09 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: BitType.java,v $
//  Revision 1.2  2001/05/25 21:05:09  cvs
//  some changes made so that the dvtg works fine. It should not affect other things
//
//  --srinivas
//
//  Revision 1.1  2000/10/25 20:50:06  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//
//
//=============================================================================

package ROM;

import java.util.HashSet;

/** This represents the <B>BitType</B> object
 * @author <I>Cindy Kong</I>
 * <a href="mailto:ckong@ittc.ukans.edu"></a>
 */
public class BitType extends Type {

    public static final Integer ONE = new Integer(1);
    public static final Integer ZERO = new Integer(0);

	private static BunchValue elements;

	/** The primary Constructor
	 * @param parentIn The parent object
	 * @return Not applicable
	 */
	public BitType(BaseObject parentIn) {
		super(parentIn);
		HashSet returnSet = new HashSet();
		returnSet.add(ZERO);
		returnSet.add(ONE);
		elements = new BunchValue(null, this, returnSet);
	}

	/** To verify that some object has correct type
	 * @param valIn Object to be tested against type 
	 * @return boolean, true if object of correct type
	 */
	public static boolean isTypeOf(Object valIn) {
		return ((valIn instanceof Integer) && 
			((Integer)valIn == ONE || (Integer)valIn == ZERO));
	}

	/** To get the set of all possible values in type
	 * Note: It may not always be possible to implement this set (infinite)
	 * @return BunchValue containing values
	 */
	public BunchValue getElements() {
		return elements;
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		return "bit";
	}
	
}

