//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: BunchValue.java,v 1.8 2000/11/13 21:49:10 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: BunchValue.java,v $
//  Revision 1.8  2000/11/13 21:49:10  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.7  2000/11/06 17:50:58  cvs
//  Additional changes made as the loader is being built. A new type
//  (component type) is added to the Universe. The types are initialized
//  in universe.
//  -- Cindy
//
//  Revision 1.6  2000/10/30 22:34:30  cvs
//  Added secondary constructors to some of the classes and some
//  accessor functions are also added 
//  --srinivas
//
//  Revision 1.5  2000/10/25 20:50:06  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.4  2000/09/18 20:41:02  cvs
//  Added correct tabbing to files. Also added CVS log info to new files
//  added by Steve Parker (Averstar).
//  -- Cindy
//
//  Revision 1.3  2000/08/30 20:39:11  cvs
//  Changed Expression class to inherit from Value and amended all
//  other classes affected by this change. -David 
//
//  Revision 1.2  2000/08/16 18:56:24  cvs
//  Added comments for documentation.
//  All files now compile.
//  -- Cindy
//
//  Revision 1.1  2000/08/11 16:37:13  cvs
//  Same for Bunch.java as for Array, Set, and Sequence. -David
//
//  Revision 1.2  2000/08/10 01:04:11  cvs
//  All object in ROM has a parent, therefore implemented the constructors
//  to take in a BaseObject parent.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:07  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.1  2000/08/09 15:43:11  cvs
//  Just adding some class skeltons for various subclasses of Expression
//
//
//=============================================================================
package ROM;

import java.util.HashSet;
import java.io.*;
/** This represents the <B>Bunch</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */

public class BunchValue 
extends Expression {

    Type type;
    HashSet elements;   //not sure about the type on this--some sort of set...
                        //literals???

	// Changed the type of the param from Item to BaseObject
	/** The primary constructor
	 * @param parent The BaseObject parent of object of this class
	 * @param typeIn The type of the contents of the bunch
	 * @param elementsIn The set of contents of the bunch
	 * @return Not applicable
	 */
	public BunchValue(BaseObject parent, Type typeIn, 
					  HashSet elementsIn) {
	    super(parent);
	    type = typeIn;
	    elements = elementsIn;
	}

	// Changed the type of param from Item to BaseObject
	/** Constructor 
	 * @param parent The BaseObject parent of object of this class
	 * @param typeIn The type of the contents of the bunch
	 * @param elemIn An object of the bunch
	 * @return Not applicable
	 */
	public BunchValue(BaseObject parent, Type typeIn, Object elmtIn) {
	    super(parent);
	    type = typeIn;
		if (elements == null)
			elements = new HashSet();
		addAnElement(elmtIn);
	}

	/** To retrieve element type
	 * @return Type representing the type of elements in the bunch
	 */
	public Type getType(){
	    return type;
	}

	/** To set element type
	 * @param typeIn Type to be set
	 * @return void
	 */
	public void setType(Type typeIn){
	    type = typeIn;
	}

	/** To return the bunch
	 * @return HashSet The set representing the bunch
	 */
	public HashSet getElementSet(){
	    return elements;
	}

	/** To set the bunch
	 * @param elementsIn The collection to set
	 * @return void
	 */
	public void setElementSet(HashSet elementsIn) {
		elements = elementsIn;
	}

	/** To add an element to the bunch
	 * @param elmtIn Element to be added
	 * @return void
	 */
	public void addAnElement(Object elmtIn){
	    elements.add(elmtIn);
	}

	/** To check if an object is found in bunch
	 * @param elmtIn Element to be checked for presence
	 * @return boolean, true if element is present
	 */
	public boolean elementPresent(Object elmtIn) {
		return elements.contains(elmtIn);
	}

	/** To union two bunch values. The resulting bunch is a new bunch.
	 * @param bunchOne First bunch value
	 * @param bunchTwo Second bunch value
	 * @return BunchValue, new bunch value obtained from the union
	 */
	public static BunchValue bunchUnion(BunchValue bunchOne, 
										BunchValue bunchTwo) {
		if (bunchOne.getParent() != bunchTwo.getParent()) {
			// To throw an error ?
			System.out.println
				("Warning: Trying to union bunch from different parents");
		}
		HashSet resultSet = new HashSet();
		resultSet.addAll(bunchOne.getElementSet());
		resultSet.addAll(bunchTwo.getElementSet());
		Type resultType = 
			Universe.chooseType(bunchOne.getType(), bunchTwo.getType());
		return new 
			BunchValue((Item)bunchOne.getParent(), resultType, resultSet);
	}

	/** To "concatenate" two bunch values. The resulting bunch is the
	 * current bunch to which elements of another bunch have been added.
	 * @param bunchOther The other bunch value
	 * @return void
	 */
	public void bunchConcat(BunchValue bunchOther) {
		if (getParent() != bunchOther.getParent()) {
			// To throw an error ?
			System.out.println
				("Warning: Trying to union bunch from different parents");
		}
		this.getElementSet().addAll(bunchOther.getElementSet());
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		// to be completed
		return null;
	}

	/** For cases when value references an item.
	 * Irrelevant here.
	 */
	public LabelledItem labelLookup(Label label2){
		return null;
	}
	public void publishROM(PrintWriter pw){
		if (toString() !=null)
			pw.print(toString());
	}


}

		

