//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: sparker@averstar.com
//
//  $Id: Component.java,v 1.5 2001/03/12 23:56:46 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Component.java,v $
//  Revision 1.5  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.4  2000/12/04 20:12:16  cvs
//  Assumption, definition and verification blocks are labelled items.
//  They are also put in the declared item list (instead of list of terms
//  as in previous version).
//  -- Cindy
//
//
//
//=============================================================================

package ROM;

import java.util.ListIterator;

/**
 * <b>Component</b>:  Represtents the type of facet know as a component.
 *
 * @author <i>Steve Parker</i> 
 * (<a href="mailto:sparker@averstar.com">sparker@averstar.com</a>).
 */
public class Component extends Facet {

    /**
     * Primary constructor.
     *
     * Assumed that first three terms of this Facet are the Assumption, 
     * Definition, and Verification Facets.
     *
     * @param parentIn The parent of this Component. 
     * @param assumptionIn The ASSUMPTION Facet.
     * @param definitionIn The DEFINITION Facet.
     * @param verificationIn The VERIFICATION Facet.
     */
    public Component(Item parentIn, 
	                 Item assumptionIn, Item definitionIn,
					 Item verificationIn) {
		super(parentIn);
		this.setTerms(null);
		this.addItem(assumptionIn);
		this.addItem(definitionIn);
		this.addItem(verificationIn);
    }

	/**
	 * Another constructor.
	 * Do not need to initialize the assumption, definition and
	 * verification facets in the constructor
	 * @param parentIn The parent of this Component. 
	 */
	public Component(Item parentIn) {
		super(parentIn);
		this.setTerms(null);
	}

    /**
     * Overrides addFacet method, so it will do nothing, thus limiting the 
     * contained facet count.
     *
     * @param facet
     */
    public void addFacet(Facet facet) {
		/*  Do nothing */
    }

	/**
	 * To set the assumption facet
	 * @param assumptionIn The ASSUMPTION item.
	 * @return void
	 */
	public void setAssumption(Item assumptionIn) {
		this.addItem(assumptionIn);
	}
	
	/**
     * Return the ASSUMPTION Facet.
     */
    public Item getAssumption() {
		ItemList itemList = this.getItems();
        LabelledItem assumption = null;
		ListIterator itemIter = itemList.listIterator();
		while (itemIter.hasNext()) {
			assumption = (LabelledItem)itemIter.next();
			if (assumption.getLabel().equals
				(new Label((Item)getParent(), "assumption")))
				return assumption;
		}
		return null;
    }

    /**
	 * To set the definition facet
	 * @param definitionIn The DEFINITION item.
	 * @return void
	 */
	public void setDefinition(Item definitionIn) {
		this.addItem(definitionIn);
	}
	
	/**
     * Return the DEFINITION Facet.
     */
    public Item getDefinition() {
		ItemList itemList = this.getItems();
		LabelledItem definition = null;
		ListIterator itemIter = itemList.listIterator();
		while (itemIter.hasNext()) {
			definition = (LabelledItem)itemIter.next();
			if (definition.getLabel().equals
				(new Label((Item)getParent(), "definition")))
				return definition;
		}
		return null;
    }

    /**
	 * To set the verification facet
	 * @param verificationIn The VERIFICATION item.
	 * @return void
	 */
	public void setVerification(Item verificationIn) {
		this.addItem(verificationIn);
	}
	
	/**
     * Return the VERIFICATION Facet.
     */
    public Item getVerification() {
		ItemList itemList = this.getItems();
		LabelledItem verification = null;
		ListIterator itemIter = itemList.listIterator();
		while (itemIter.hasNext()) {
			verification = (LabelledItem)itemIter.next();
			if (verification.getLabel().equals
				(new Label((Item)getParent(), "verification")))
				return verification;
		}
		return null;
 
    }

    /**
     * labelLookup - call labelLookup on contained Facet's.
     *
     * @param labelIn Label which is being searched for.
     * @return LabelledItem found, which corresponds to label.
     */
	// ckong March 8, 2001 - the labelLookup should be the same for
	// Component and Facet !
    //public LabelledItem labelLookup(Label labelIn) {
	//	LabelledItem item;
	//	ListIterator iterator = getItems().listIterator();
	//	LabelledItem toReturn;
	//	while (iterator.hasNext()) {
	//		item = (LabelledItem)iterator.next();
	//		if (item.getValue() != null) {
	//			toReturn = ((Facet)item.getValue()).labelLookup(labelIn);
	//			if (toReturn != null)
	//				return toReturn;
	//		}
	//	}
	//	return null; // may rather want to throw an exception here
    //}
	

}
