//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: DataLiteral.java,v 1.7 2001/03/12 23:56:46 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: DataLiteral.java,v $
//  Revision 1.7  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.6  2001/03/06 02:15:14  cvs
//  Added some accessor methods that are used in DVTG to the ROM classes
//
//  --srinivas
//
//  Revision 1.5  2001/02/01 22:34:29  cvs
//  Added some accessor methods to the classes which are used in DVTG
//  --srinivas
//
//  Revision 1.4  2000/12/03 23:59:45  cvs
//  Added the new types of array, set and sequence. The methods
//  getType  and alreadyDeclared are also added to get a type that has
//  been previously declared. 
//  --srinivas
//
//  Revision 1.3  2000/08/17 20:48:22  cvs
//  Added some members to the literal classes.
//  Modified the lookup algorithm in Universe to remove an error.
//  -- Cindy
//
//  Revision 1.2  2000/08/10 01:04:12  cvs
//  All object in ROM has a parent, therefore implemented the constructors
//  to take in a BaseObject parent.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:07  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.1  2000/08/09 15:44:15  cvs
//  Just adding some class skeltons for various subclasses of Expression
//
//
//=============================================================================
package ROM;
import java.io.*;
/** This represents the <B>DataLiteral</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */

public class DataLiteral 
extends Literal {

	Number numericData= null;	
	Boolean booleanData = null;
	String stringdata = null;

	/** The primary constructor
	 * @param numericIn Numeric value of the data
	 * @param parent The BaseObject parent of an object of this class
	 * @return Not applicable
	 */
	public DataLiteral(BaseObject parent, Number numericIn) {
		super(parent);
		numericData = numericIn;
	}

	
	/** The secondary constructor 
	 * @param BaseObject 
	 */
	public DataLiteral(BaseObject parent){
		super(parent);
	}

	/** Another constructor
	 * @param booleanIn Boolean value of the data
	 * @param parent The BaseObject parent of an object of this class
	 * @return Not applicable
	 */
	public DataLiteral(BaseObject parent, Boolean booleanIn) {
		super(parent);
		booleanData = booleanIn;
	}

	/** To get the value that is not null
	 * @return Object
	 */
	public Object getData() {
		if (numericData != null)
			return numericData;
		return booleanData;
	}

	/** To get the integer value
	 * @return integer
	 */
	public Integer getIntValue() {
		return (Integer)numericData;
	}

	/** To get the double value
	 * @return double
	 */
	public Float getFloatValue() {
		return (Float)numericData;
	}

	/**To get the double value
	 * @return double
	 */
	public Double getDoubleValue() {
		return (Double)numericData;
	}

	/** To get the boolean value
	 * @return boolean
	 */
	public Boolean getBooleanValue() {
		return booleanData;
	}

	/** To get the String data value
	 * @return String
	 */
	public String getStringData(){
		return stringdata;
	}

	/** To set the String data value
	 * @param String
	 */
	public void setStringData(String stringIn){
		stringdata = stringIn;
	}


	/** To get the numeric value
	 * @return Number
	 */
	public Number getNumericData(){
		return numericData;
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		// to be completed
		return null;
	}

	public void publishROM(PrintWriter pw){
		String str;
		if (numericData != null){
			if (numericData instanceof Integer ){
				str = ((Integer)numericData).toString();
				pw.print(str);
			}else if (numericData instanceof Float){
				str = ((Float)numericData).toString();
				pw.print(str);
			}else if (numericData instanceof Double){
				str = ((Double)numericData).toString();
				pw.print(str);
			}
		}else if (booleanData != null){
			str = booleanData.toString();
			pw.print(str);
		}else
			pw.print(stringdata);
		return ;
	}

}

		

