//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: LabelledItem.java,v 1.8 2001/05/24 20:13:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: LabelledItem.java,v $
//  Revision 1.8  2001/05/24 20:13:47  cvs
//  Some small changes made in each of these files.
//  -- Cindy
//
//  Revision 1.7  2001/02/07 20:52:20  cvs
//  There was an error causing an infinite loop when doing label lookup in
//  use clauses. Modifications were made in labelLookup methods to remove
//  error.
//  -- Cindy
//
//  Revision 1.6  2000/12/04 19:39:38  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  Class Item is now an abstract class.
//  -- Cindy
//
//  Revision 1.5  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.4  2000/10/25 20:50:07  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.3  2000/09/16 19:48:16  cvs
//  Revamped ROM class hierarchy.  Committed partially skelatal
//  classes that successfully compile. 
//
//  Revision 1.2  2000/08/22 18:40:17  cvs
//  First Commit by Steve Parker, Averstar.  Basic Structure of
//  classes setup for ROM. 
//
//  Revision 1.1  2000/08/09 04:29:54  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//
//=============================================================================
package ROM;

/** This represents the <B>LabelledItem</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class LabelledItem 
extends Item {

	private Label label;
	
	/** The primary constructor
	 * @param parent BaseObject representing the parent
	 * @param labelIn Label representing label of item
	 * @param typeIn Type representing the type
	 * @param valueIn Value representing value of the item
	 * @param stringIn String representation of item
	 * @return Not applicable
	 */
	public LabelledItem(BaseObject parent, Label labelIn, 
						Type typeIn, Expression valueIn, 
						String stringIn) {
		super(parent, typeIn, valueIn, stringIn);
		label = labelIn;
	}
    
	/** Another constructor
	 * @param parent BaseObject representing the parent
	 * @param labelIn Label representing label of item
	 * @param typeIn Item representing the type
	 * @param stringIn String representation of item
	 * @return Not applicable
	 */
	public LabelledItem(BaseObject parent, Label labelIn, 
						Type typeIn, String stringIn){
		super(parent, typeIn, null, stringIn);
		label = labelIn;
	}

	/** To get the label
	 * @return Label the label of the item
	 */
	public Label getLabel() {
		return label;
	}

	/** To set the label
	 * @param labelIn The label to be set
	 * @return void
	 */
	public void setLabel(Label labelIn) {
		label = labelIn;
	}

	/** To get the name (string of the label) of item
	 * @return String corresponding to label
	 */
	public String toString() {
		return getLabel().toString();
	}

	/** To look up a label. If the item has the same label return the
	 * item else look for it in the parent
	 * @return Item corresponding to label
	 */
    public LabelledItem labelLookup(Label label2){

		if (getLabel().equals(label2)) 
			return this;
		
		return getParent().labelLookup(label2);
	}
}
	       
