//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//
//  $Id: OperatorReference.java,v 1.3 2001/05/30 16:31:58 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: OperatorReference.java,v $
//  Revision 1.3  2001/05/30 16:31:58  cvs
//  Added some accessor methods to the classes. The methods are used by the Moml to Rosetta converter.
//
//  --Srinivas
//
//  Revision 1.2  2001/05/24 20:13:47  cvs
//  Some small changes made in each of these files.
//  -- Cindy
//
//  Revision 1.1  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//
//
//=============================================================================
package ROM;

import java.io.*;
import java.util.*;
/** This represents the <B>OperatorReference</B> object
 * @author <I> Cindy Kong </I> (ckong@ittc.ukans.edu)
 */

public class OperatorReference 
extends FunctionReference {

	Type opType;

	/** Primary constructor
	 * @param parent The BaseObject parent
	 * @param itemIn The function item being referred
	 * @param typeIn The type of the operator
	 * @return Not applicable
	 */
	public OperatorReference(BaseObject parent, Item itemIn, Type typeIn) {
		super(parent, itemIn);
		opType = typeIn;
	}
	
	/**The secondary construtor
	 * @param parent The BaseObject parent
	 * @param itemIn The function item being referred
	 * @return Not applicable
	 public OperatorReference(BaseObject parent, Item itemIn){
		super(parent, itemIn);
	}
	*/
	
	/** To get the type of the referenced operator
	 * @return opType, the type
	 */
	public Type getOperatorType() {
		return opType;
	}

	/** To set the type of the referenced operator
	 * @param typeIn The Type to set
	 * @return void
	 */
	public void setOperatorType(Type typeIn) {
		opType = typeIn;
	}

	/** To get the string of the name of the item referred
	 * @return String representing the string of this object
	 */
	public String toString() {
		String toReturn = ((LabelledItem)getItemReferred()).toString();
		if (!getParameters().isEmpty()) {
			ListIterator paramIterator = getParameters().listIterator();
			Expression paramReference;
			toReturn.concat("(");
			while (paramIterator.hasNext()) {
				if (paramIterator.hasPrevious())
					toReturn.concat(", ");
				paramReference = (Expression)paramIterator.next();
				toReturn.concat(paramReference.toString());
			}
			toReturn.concat(")");
		}
		return toReturn;
	}
	
	

}

		

