//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//
//  $Id: ParameterItem.java,v 1.5 2001/05/30 16:31:58 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ParameterItem.java,v $
//  Revision 1.5  2001/05/30 16:31:58  cvs
//  Added some accessor methods to the classes. The methods are used by the Moml to Rosetta converter.
//
//  --Srinivas
//
//  Revision 1.4  2001/05/24 20:13:47  cvs
//  Some small changes made in each of these files.
//  -- Cindy
//
//  Revision 1.3  2001/03/21 00:07:11  cvs
//  Added the "design" mode for parameters.
//  -- Cindy
//
//  Revision 1.2  2000/11/06 17:50:58  cvs
//  Additional changes made as the loader is being built. A new type
//  (component type) is added to the Universe. The types are initialized
//  in universe.
//  -- Cindy
//
//  Revision 1.1  2000/10/25 20:50:07  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//
//
//=============================================================================
package ROM;

/** This represents the <B>LabelledItem</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class ParameterItem 
extends LabelledItem {

	public static final int IN = 0;
	public static final int OUT = 1;
	public static final int INOUT = 2;
	public static final int DESIGN = 3;
	public static final int NOMODE = 4;
	
	private int mode;

	/** The primary constructor
	 * @param parent BaseObject representing the parent
	 * @param labelIn Label representing label of item
	 * @param typeIn Type representing the type
	 * @param valueIn Value representing value of the item
	 * @param modeIn Int representing mode of parameter
	 * @param stringIn String representation of item
	 * @return Not applicable
	 */
	public ParameterItem(BaseObject parent, Label labelIn, 
						 Type typeIn, Expression valueIn,
						 int modeIn, String stringIn) {
		super(parent, labelIn, typeIn, valueIn, stringIn);
		mode = modeIn;
	}
    
	/** Another constructor
	 * @param parent BaseObject representing the parent
	 * @param labelIn Label representing label of item
	 * @param typeIn Item representing the type
	 * @param modeIn Int representing mode of parameter
	 * @param stringIn String representation of item
	 * @return Not applicable
	 */
	public ParameterItem(BaseObject parent, Label labelIn, 
						 Type typeIn, int modeIn, String stringIn){
		super(parent, labelIn, typeIn, null, stringIn);
		mode = modeIn;
	}

	/** To get the mode
	 * @return int, the mode of the item
	 */
	public int getMode() {
		return mode;
	}

	/** To check if mode is IN
	 * @return true if mode in In
	 */
	public boolean isIn() {
		return (getMode() == IN);
	}

	/** To check if mode is OUT
	 * @return true if mode in Out
	 */
	public boolean isOut() {
		return (getMode() == OUT);
	}

	/** To check if mode is INOUT
	 * @return true if mode in InOut
	 */
	public boolean isInOut() {
		return (getMode() == INOUT);
	}

	/** To check if mode is DESIGN
	 * @return true if mode in Design
	 */
	public boolean isDesign() {
		return (getMode() == DESIGN);
	}

	/** To set the mode
	 * @param modeIn The mode to be set
	 * @return void
	 */
	public void setMode(int modeIn) {
		mode = modeIn;
	}

	/**To get the mode in a string format
	 * @param void
	 * @return String
	 */
	public String getStringMode(){
		if (mode==0){
			return "in";
		}else if (mode == 1){
			return "out";
		}else if (mode == 2){
			return "inout";
		}else if (mode == 3){
			return "design";
		}else
			return null;
	}
	
}
	       
