//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: Type.java,v 1.4 2000/11/17 23:14:14 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Type.java,v $
//  Revision 1.4  2000/11/17 23:14:14  cvs
//  Added the method labelLookup in some classes and changed the same method in others
//  --srinivas
//
//  Revision 1.3  2000/10/25 20:50:08  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.2  2000/09/16 19:48:16  cvs
//  Revamped ROM class hierarchy.  Committed partially skelatal
//  classes that successfully compile. 
//
//  Revision 1.1  2000/08/16 18:56:24  cvs
//  Added comments for documentation.
//  All files now compile.
//  -- Cindy
//
//
//
//=============================================================================
package ROM;

import java.util.HashSet;

/** This represents the <B>Type</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public abstract class Type 
extends BaseObject {	

	/** The primary Constructor
	 * @param parentIn The BaseObject representing the parent
	 * @return Not applicable
	 */
	public Type(BaseObject parentIn) {
		super(parentIn);
	}
	
	/** To get the set of all possible values of type
	 * Note: It may not always be possible to implement this set (infinite)
	 * @return BunchValue containing values of the type
	 */
	public abstract BunchValue getElements();

    /** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public abstract String toString();

	
	//Added by srinivas on Nov 10th , 2000
	// This function returns null as these are no items declared
	public LabelledItem labelLookup(Label label){
		return null;
	}

}
