//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_AndExpressionTail.java,v 1.7 2001/03/12 23:54:36 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_AndExpressionTail.java,v $
//  Revision 1.7  2001/03/12 23:54:36  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.6  2000/08/01 22:48:44  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.5  2000/03/23 00:22:45  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.4  2000/02/10 00:37:08  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.3  2000/02/08 18:42:19  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes (with the help
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the object model.
//  -- Cindy.
//
//  Revision 1.2  1999/11/05 23:43:08  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:26  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:49:07  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.3  1999/07/14 23:23:25  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.2  1999/06/30 22:19:51  rkamath
//  Compeleted the object model coding for the expression grammar
//  The expression objects now inherit from a AbstractExpression class
//
//  Revision 1.1  1999/06/30 02:18:55  rkamath
//  Added a lot more files dealing with the expression OM.
//  Also modified some previous classes
//  Now the ROOT in the hierarchy is the SOM_Unifier interface.
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;

/** This represents the <B>AndExpressionTail</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_AndExpressionTail
extends SOM_AbstractExpressionTail {

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_AndExpressionTail() {
		super();
	}

	/** To set the operator
	 * @param opIn The SOM_DyadicOperator to be set
	 * @return void
	 */
	public void setOperator(SOM_DyadicOperator opIn) {
		super.setOperator(opIn);
	}

	/** To get the operator
	 * @return The SOM_DyadicOperator for this expression
	 */
	public SOM_DyadicOperator getOperator() {
		return super.getOperator();
	}

	/** To set the relationalTermExpression
	 * @param termIn The SOM_RelationalTermExpression to be set
	 * @return void
	 */
	public void	setRelationalTermExpression(SOM_RelationalTermExpression 
											termIn) {
		setExpression((SOM_AbstractExpression)termIn);
	}

	/** To get the relationalTermExpression
	 * @return The SOM_RelationalTermExpression for this expression
	 */
	public SOM_RelationalTermExpression getRelationalTermExpression() {
		return (SOM_RelationalTermExpression)getExpression();
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		super.getOperator().publishSOM(pw);
		getExpression().publishSOM(pw);
	}

	/** The publishPVS routine for this class
	 * @return void
	 */
	public void publishPVS(PrintWriter pw) {
		super.getOperator().publishPVS(pw);
		getExpression().publishPVS(pw);
	}
}
