//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_ArrayListing.java,v 1.11 2000/11/18 00:18:26 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_ArrayListing.java,v $
//  Revision 1.11  2000/11/18 00:18:26  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.10  2000/10/30 22:13:22  cvs
//  The M__Value  function is implemented for the terms(SOM_LabelledExpression) and it returns a ROM Expression.
//  --srinivas
//
//  Revision 1.9  2000/08/01 22:48:45  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.8  2000/06/22 17:22:38  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.7  2000/04/28 19:34:04  cvs
//  Added some more of typechecking. Added parsing of the $ operator and constant literal.
//  -- Cindy
//
//  Revision 1.6  2000/04/06 19:13:18  cvs
//  Type checking is done for declaration of variables (simple types), for declaration of new types and for use of basic operators.
//   -- Cindy
//
//  Revision 1.5  2000/03/23 00:22:45  cvs
//  Huge commit! Added all the essential accessor methods for the following classes
//  - Krishna and Makarand
//
//  Revision 1.4  2000/02/10 00:37:09  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.3  2000/02/08 18:42:19  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes (with the help
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the object model.
//  -- Cindy.
//
//  Revision 1.2  2000/02/02 20:23:46  cvs
//  Committing the class files I have modified due to changes made to the grammar.
//  -- Cindy
//
//  Revision 1.1  1999/10/27 20:43:27  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:49:12  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.1  1999/07/22 22:53:14  rkamath
//  Lot of changes especially in terms of adding accessors to the OM
//  This does not compile completely, the toString() of SOM_FormalIdentifier
//  is to be completed. This commit is purely for records
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;
import ROM.*;
import util.ListEnumeration;

/** This represents the <B>ArrayListing</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_ArrayListing
extends SOM_AbstractExpression
implements SOM_Factor{

	private SOM_ExpressionList exprList = new SOM_ExpressionList();

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_ArrayListing() {
		super();
	}

	/** To add to the expressionList
	 * @param item The SOM_Expression to be added
	 * @return void
	 */
	public void addToExpressionList(SOM_Expression item) {
		exprList.addElement(item);
	}

	/** 
	 * @param declsIn The SOM_DeclarationList to be set
	 * @return void
	 */
	public void setTypeDeclaration(SOM_TypeDeclaration currDecl) {

		SOM_TreeNodeList toReturn = new SOM_TreeNodeList();
		SOM_TreeNode node;

		node = new SOM_TreeNode((SOM_TypeDeclaration)currDecl);
		toReturn.addElement(node);
		setTypeList(toReturn);
	}


	/** To do the semcheck. Overrides that in SOM_AbstractExpression
	 * Note that it assumes a semChecked expression - consistent with
	 * the semChecks elsewhere..
	 * @return void
	 */
	public void semCheck() {
		//setTypeList(expression.getTypeList());
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		//expression.resolveFinalType();
		//setResolvedType(expression.getResolvedType());
		setResolvedType(((SOM_TreeNode)getTypeList().first()).
						getResultType()); 
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		//expression.resolveFinalType(typeIn);
		//setResolvedType(expression.getResolvedType());
		SOM_TypeDeclaration manifestType = 
			((SOM_TreeNode)getTypeList().first()).getResultType();
		if (SOM_TypeDeclaration.resolveFinalType(manifestType, 
												 typeIn))
			setResolvedType(manifestType);
			
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
		//return expression.getNames();
		return null;
	}

	/** To retrieve the listEnumeration of the expressionTailList
	 * @return The ListEnumeration corresponding to the expressionTailList
	 */
	public ListEnumeration arrays() {
		return exprList.elements();
	}


	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,
							   BaseObject parent,Universe root){
		ArrayValue arrVal = new ArrayValue(parent);
		ExpressionList exprList = new ExpressionList();
		ListEnumeration exprs = arrays();
		if (exprs !=null){
			while (exprs.hasMoreElements()){
				Object obj = exprs.nextElement();
				if (obj instanceof SOM_Expression){
					Expression romExpr =
						((SOM_Expression)obj).
						M__Value(romFactory,arrVal,root);
					exprList.addItem(romExpr);
				}
			}
			arrVal.setElements(exprList);
		}
		//set the type
		return (Expression)arrVal;
	}


	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
	    ListEnumeration exprListEnum = arrays();
	    SOM_Expression expr;

	    while(exprListEnum.hasMoreElements()){
			pw.print("[ ");
			expr = (SOM_Expression)(exprListEnum.nextElement());
			expr.publishSOM(pw);
         	pw.print("] ");
	    }
	}

	/** dummy functions added by krishna and makarand on Mar 7, 2000
     */
		public SOM_Label getLabel() {
				return (SOM_Label)null;
		}
		
		public SOM_Expression getLabelExpression() {
				return (SOM_Expression)null;
		}

		/** The publishPVS routine for this class
		 * @return void
		 * Method to be implmented later - makarand.
		 */
		public void publishPVS(PrintWriter pw) {
				publishSOM(pw);
		}

}


