//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Srinivas akki@ittc.ukans.edu
//
//  $Id: SOM_AtTickExpression.java,v 1.13 2001/03/12 23:54:36 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_AtTickExpression.java,v $
//  Revision 1.13  2001/03/12 23:54:36  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.12  2000/11/27 23:49:55  cvs
//  Previously was not taking the tick and at operators into
//  consideration while building the ROM so made changes to include
//  these operators. 
//  --srinivas
//
//  Revision 1.11  2000/11/18 00:18:26  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.10  2000/10/30 22:13:23  cvs
//  The M__Value  function is implemented for the
//  terms(SOM_LabelledExpression) and it returns a ROM Expression. 
//  --srinivas
//
//  Revision 1.9  2000/08/10 00:38:29  cvs
//  added publishPVS routine in Formal name class and modified the
//  exisiting publishPVS routines in AttickExpression and Identifier
//  to remove an error during conversion of tick to underscore - makarand
//
//  Revision 1.8  2000/08/09 23:37:55  cvs
//  added publishPVS routines for these classes - Makarand
//
//  Revision 1.7  2000/08/01 22:48:45  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.6  2000/06/22 17:22:39  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.5  2000/04/20 22:59:13  cvs
//  Added some accessor methods to this class
//  - Krishna
//
//  Revision 1.4  2000/04/09 01:50:02  cvs
//   The identifier had a space printed in the publishSOM which was
//   not required for the tick and the at operators so changing the
//   classes and the grammar had the tick and single quote
//   interchanged so modifing the grammar also 
//  --srinivas
//
//  Revision 1.3  2000/04/08 04:05:38  cvs
//  The tick and the single quote symbols were interchanged so
//  modified the grammar and the classes to correct that . 
//  --srinivas
//
//  Revision 1.2  2000/04/06 19:13:18  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.1  2000/04/05 07:25:46  cvs
//  The at and tick operators have been added .There were some changes
//  to the predefined unary function name .The grammar has been
//  changed and some classes ahve been added like SOM_AtOperator
//  SOM_TickOperator and SOM_AtTickExpression. 
//  --srinivas
//
//
//
//=============================================================================
package SOM;
import util.ListEnumeration;

import java.io.PrintWriter;
import util.*;
import SOM.*;
import SYM.*;
import ROM.*;
//import java.lang.Error;
//import java.util.NoSuchElementException;
import java.util.StringTokenizer;

/** This represents the <B>AtTickExpression</B> object
 * @author <I>Srinivas</I> (akki@ittc.ukans.edu)
 */

public  class SOM_AtTickExpression
extends SOM_AbstractExpression 
implements SOM_Factor
{
    private SOM_Name name;
    private SOM_Identifier timeId = null;
    private SOM_Identifier valueId = null;
    private SOM_TickOperator tickOp = null;
    private SOM_AtOperator atOp = null;
    private SOM_DyadicOperator op = null;
    private SOM_Literal literal = null;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_AtTickExpression() {
		super();
	}

	/** To set the name
	 * @param nameIn   SOM_Name to be set
	 * @return void
	 */
    public void setName(SOM_Name nameIn){
		name=nameIn ;
		
	}

	/** To get the name
	 * @return The SOM_Name for this expression
	 */
	public SOM_Name getName(){
		return name;
	}

	/** To set the tickOperator
	 * @param tickOpIn   SOM_TickOperator to be set
	 * @return void
	 */
    public void setTickOperator(SOM_TickOperator tickOpIn){
		tickOp = tickOpIn ;
	}

	/** To get the tickOperator
	 * @return The SOM_TickOperator for this expression
	 */
  public SOM_TickOperator getTickOperator(){
		return tickOp;
	}
	
	/** To set the atOperator
	 * @param atOpIn   SOM_AtOperator to be set
	 * @return void
	 */
    public void setAtOperator(SOM_AtOperator atOpIn){
		atOp = atOpIn ;
	}

	/** To get the atOperator
	 * @return The SOM_AtOperator for this expression
	 */
	public SOM_AtOperator getAtOperator(){
		return atOp;
	}
	
	/** To set the timeIdentifier
	 * @param timeIdIn   SOM_Identifier to be set
	 * @return void
	 */
    public void setTimeIdentifier(SOM_Identifier timeIdIn){
		timeId = timeIdIn ;
	}

	/** To get the timeIdentifier
	 * @return The SOM_Identifier for this expression
	 */
	public SOM_Identifier getTimeIdentifier(){
		return timeId;
	}

	/** To set the valueIdentifier
	 * @param valueIdIn   SOM_Identifier to be set
	 * @return void
	 */
	public void setValueIdentifier(SOM_Identifier valueIdIn){
		valueId = valueIdIn ;
	}

	/** To get the valueIdentifier
	 * @return The SOM_Identifier for this expression
	 */
	public SOM_Identifier getValueIdentifier(){
		return valueId;
	}

	/** To set the operator
	 * @param opIn SOM_DyadicOperator to be set
	 * @return void
	 */
	public void setOperator(SOM_DyadicOperator opIn){
		op = opIn ;
	}

	/** To get the operator
	 * @return The SOM_DyadicOperator for this expression
	 */
	public SOM_DyadicOperator getOperator(){
		return op;
	}

	/** To set the literal
	 * @param literalIn   SOM_Literal to be set
	 * @return void
	 */
	public void setLiteral(SOM_Literal literalIn){
		literal=literalIn;
	}

	/** To get the literal
	 * @return The SOM_Literal for this expression
	 */
	public SOM_Literal getLiteral(){
		return literal;
	}

	/** dummy functions added by krishna and makarand on Mar 7, 2000
     */
	public SOM_Label getLabel() {
		return (SOM_Label)null;
	}
		
	public SOM_Expression getLabelExpression() {
		return (SOM_Expression)null;
	}

	/** To set the tree of declarations
	 * @param treeIn The SOM_TreeNodeList to be set
	 * @return void
	 */
	public void setTree(SOM_TreeNodeList treeIn) {
		setTypeList(treeIn);
	}

	/** To do the sem check. A call to this function SHOULD set its typeList
	 * Note that this will be inherited by each child and is generic enough
	 * for most kinds of "expression" classes. Note that it implicitly
	 * assumes that the strongerExpression has been set to a valid object
	 * if not it will throw a NullPointerException
	 * @return void
	 */
	public void semCheck() {	
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. This is generic enough and should be
	 * overridden at the leaves of the expression tree!
	 * @return void
	 */
	public void resolveFinalType() {
		name.resolveFinalType();
		setResolvedType(name.getResolvedType());
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		name.resolveFinalType(typeIn);
		setResolvedType(name.getResolvedType());
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
	    return name.getNames();
	}	

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,
							   BaseObject parent,Universe root){
		SOM_Name name = getName();
		Expression expr = name.M__Value(romFactory,parent,root);
		if (tickOp != null){
			Formula retExpr = new Formula(parent);
			retExpr.setLeftOp(expr);
			OperatorReference tickRef = new 
				OperatorReference(retExpr, root.labelLookup("'"), null);
			retExpr.setOperator(tickRef);
			return retExpr;
		}else if (atOp !=null){
			Formula retExpr = new Formula(parent);
			retExpr.setLeftOp(expr);
			OperatorReference atRef = new 
				OperatorReference(retExpr, root.labelLookup("@"), null);
			retExpr.setOperator(atRef);
			Formula valueFormula = new Formula(retExpr);
			valueFormula.setLeftOp(timeId.M__Value(romFactory,retExpr,root));
			if (valueId != null) {
				valueFormula.setRightOp
					(valueId.M__Value(romFactory, retExpr, root));
				OperatorReference ref = new OperatorReference
					(valueFormula, root.labelLookup(op.toString()), 
					 root.getType(op.getResolvedType().toString()));
				valueFormula.setOperator(ref);
				retExpr.setRightOp(valueFormula);
			} else {
				valueFormula.setRightOp
					(literal.M__Value(romFactory, retExpr, root));
				OperatorReference ref = new OperatorReference
					(valueFormula, root.labelLookup(op.toString()), 
					 root.getType(op.getResolvedType().toString()));
				valueFormula.setOperator(ref);
				retExpr.setRightOp(valueFormula);
			}
			return retExpr;
		}else {
			return expr;
		}
	}
	
	/** publishSOM routine needed to print the object module
	 * @return void
	 */
	public void publishSOM(PrintWriter pw){
		name.publishSOM(pw);
		if (tickOp !=null){
			pw.print("'");
		}
		if (atOp !=null){
			pw.print("@");
			timeId.publishSOM(pw);
			if (op !=null){
				op.publishSOM(pw);
				if (valueId != null){
					valueId.publishSOM(pw);
				}else {
					literal.publishSOM(pw);
				}
			}
		}
	}

	/** publishPVS routine needed to print the object module
	 * @return void
	 */
	public void publishPVS(PrintWriter pw){
		name.publishPVS(pw);
		if (tickOp !=null){
			pw.print("_");
		}
		if (atOp !=null){
			pw.print("@");
			timeId.publishPVS(pw);
			if (op !=null){
				op.publishPVS(pw);
				if (valueId != null){
					valueId.publishPVS(pw);
				}else {
					literal.publishPVS(pw);
				}
			}
		}
	}
}
