package SOM;

import util.ListEnumeration;
import java.io.PrintWriter;
import ROM.*;
import java.util.*;

/** This represents the <B>SOM_ConstructedTypeDeclarationStatement</B> object
 * @author <I>Brandon Morel</I> (morel@ittc.ku.edu)
 */
public class SOM_ConstructedTypeDeclarationStatement 
extends SOM_BaseClass {

    private SOM_TypeDeclaration typeDecl = null;
    private SOM_ConstructorList constrList = new SOM_ConstructorList();
	
    /** The primary Constructor
     * @return Not applicable
     */
    public SOM_ConstructedTypeDeclarationStatement()
    {
	super();
    }

    /** To set the type declaration for the type
     * @param typeDeclIn The SOM_TypeDeclaration representing the type
     * @return void
     */
    public void setType(SOM_TypeDeclaration typeDeclIn) {
	typeDecl = typeDeclIn;
    }

    /** To get the type declaration for the type
     * @return The SOM_TypeDeclaration representing the type
     */
    public SOM_TypeDeclaration getType() {
	return typeDecl;
    }
 
    /** To add a constructor
     * @param constrIn The SOM_Constructor to be added
     * @return void
     */
    public void addToConstructorList(SOM_Constructor constrIn) {
	constrList.addElement(constrIn);
    }
  
    /** to retreive the constructor list
     * @return SOM_ConstructorList
     */
    public SOM_ConstructorList getConstructorList() {
	return constrList;
    }
    
    /** To retreive an enumeration of the constructors
     * @return A ListEnumeration corresponding to the constructors stored
     * in the constrList
     */
    public ListEnumeration constructors() {
	return constrList.elements();
    }

    /** The buildROM method for this item
     * @param romFactory, Factory object used to create objects in ROM
     * @param root, Universe object 
     * @param parentIn, BaseObject parent of item
     * @return void, because I will be generating these functions
     */
    public void buildROM(Factory romFactory, BaseObject parentIn,
			 Universe root){
	ListEnumeration constrEnum = constructors();
	while (constrEnum.hasMoreElements()) {
	    SOM_Constructor constructorItem = (SOM_Constructor)constrEnum.nextElement();
	    constructorItem.buildROM(romFactory, parentIn, root);
	}   
    }

    /** The publishSOM routine for this class
     * @return void
     */
    public void publishSOM(PrintWriter pw){
	
	ListEnumeration constructorEnum = constructors();
	SOM_Constructor constructorItem;
	pw.print(" data ");
	pw.println();
	constructorItem = (SOM_Constructor)constructorEnum.nextElement();
	constructorItem.publishSOM(pw);
	while (constructorEnum.hasMoreElements()) {
	    pw.print(" | ");
	    pw.println();
	    constructorItem = (SOM_Constructor)constructorEnum.nextElement();
	    constructorItem.publishSOM(pw);
	}
    }
}
