package SOM;

import util.Link;
import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;
import java.io.*;
import util.List;

/** This represents the <B>Constructor</B> object
 * @author <I>Brandon Morel</I> (morel@ittc.ku.edu)
 */
public class SOM_Constructor
    extends SOM_Declaration {

    private SOM_Identifier id;
    private SOM_ObserverListing observerList = null;
    private SOM_Recognizer recognizer = null;
    private SOM_TypeDeclaration returnType;

    /** The primary constructor
     * @return Not applicable
     */
    public SOM_Constructor(SOM_TypeDeclaration typeIn) {
	super();
	returnType = typeIn;
    }

    /** To set the identifier
     * @param idIn The SOM_Identifier to be set
     * @return void
     */
    public void setIdentifier(SOM_Identifier idIn) {
	id = idIn;
    }		 
	 
    /** To get the identifier
     * @return The SOM_Identifier for this constructor
     */
    public SOM_Identifier getIdentifier() {
	return id;
    }

    /** To set the type declaration for the type
     * @param typeDeclIn The SOM_TypeDeclaration representing the type
     * @return void
     */
    public void setType(SOM_TypeDeclaration typeDeclIn) {
	returnType = typeDeclIn;
    }

    /** To get the type declaration for the type
     * @return The SOM_TypeDeclaration representing the type
     */
    public SOM_TypeDeclaration getType() {
	return returnType;
    }

    /** To set the observerListing - implements the function from
     * Constructor
     * @param observerIn The SOM_ObserverListing object to be set
     * @return void
     */
    public void setObserverListing(SOM_ObserverListing observerIn) {
	observerList = observerIn;
    }

    /** To get the observerListing - implements the function from
     * Constructor
     * @return The SOM_ObserverListing object
     */
    public SOM_ObserverListing getObserverListing() {
	return observerList;
    }

    /** To set the recognizer
     * @param recognizerIn The SOM_Recognizer object to be set
     * @return void
     */
    public void setRecognizer(SOM_Recognizer recognizerIn) {
  	recognizer = recognizerIn;
    }

    /** To get the recognizer
     * @return The SOM_Recognizer object
     */
    public SOM_Recognizer getRecognizer() {
    	return recognizer;
    }


    /** To create the expression for a function call parameter
     * @param inId the SOM_Identifier the id of the parameter
     * @return SOM_Expression
     */
    public SOM_Expression createInputValue(SOM_Identifier inId) {
	SOM_AtTickExpression InputVariable = new SOM_AtTickExpression();
	InputVariable.setName((SOM_Name)inId);
	SOM_ExpTerm obsExpTerm = new SOM_ExpTerm();
	obsExpTerm.setFactor((SOM_Factor)InputVariable);
	SOM_Term obsTerm = new SOM_Term();
	obsTerm.setExpTerm(obsExpTerm);
	SOM_SimpleExpression obsSimpleExpr = new SOM_SimpleExpression();
	obsSimpleExpr.setTerm(obsTerm);
	SOM_RelationalTermExpression obsRelTermExpr = new SOM_RelationalTermExpression();
	obsRelTermExpr.setSimpleExpression(obsSimpleExpr);
	SOM_AndExpression obsAndExpr = new SOM_AndExpression();
	obsAndExpr.setRelationalTermExpression(obsRelTermExpr);
	SOM_ImpliesExpression obsImpliesExpr = new SOM_ImpliesExpression();
	obsImpliesExpr.setAndExpression(obsAndExpr);
	SOM_TermExpression obsTermExpr = new SOM_TermExpression();
	obsTermExpr.setImpliesExpression(obsImpliesExpr);
	SOM_Expression obsExpr = new SOM_Expression();
	obsExpr.setTermExpression(obsTermExpr);
	return obsExpr;
    }

    /** To create the expression for a function call
     * @param callVariables the SOM_Expression the parameters of the function
     * @return SOM_Expression
     */
    public SOM_Expression createFunctionCall(SOM_Expression callVariables) {
	SOM_AssociationListing obsAssocList = new SOM_AssociationListing();
	obsAssocList.addToList(callVariables);
	SOM_FormalName obsFormalName = new SOM_FormalName();
	obsFormalName.setName((SOM_Name)getRecognizer().getIdentifier());
	obsFormalName.setAssociationListing(obsAssocList);
	SOM_AtTickExpression InputVariable = new SOM_AtTickExpression();
	InputVariable.setName(obsFormalName);
	SOM_ExpTerm obsExpTerm = new SOM_ExpTerm();
	obsExpTerm.setFactor((SOM_Factor)InputVariable);
	SOM_Term obsTerm = new SOM_Term();
	obsTerm.setExpTerm(obsExpTerm);
	SOM_SimpleExpression obsSimpleExpr = new SOM_SimpleExpression();
	obsSimpleExpr.setTerm(obsTerm);
	SOM_RelationalTermExpression obsRelTermExpr = new SOM_RelationalTermExpression();
	obsRelTermExpr.setSimpleExpression(obsSimpleExpr);
	SOM_AndExpression obsAndExpr = new SOM_AndExpression();
	obsAndExpr.setRelationalTermExpression(obsRelTermExpr);
	SOM_ImpliesExpression obsImpliesExpr = new SOM_ImpliesExpression();
	obsImpliesExpr.setAndExpression(obsAndExpr);
	SOM_TermExpression obsTermExpr = new SOM_TermExpression();
	obsTermExpr.setImpliesExpression(obsImpliesExpr);
	SOM_Expression obsExpr = new SOM_Expression();
	obsExpr.setTermExpression(obsTermExpr);
	return obsExpr;
    }
	
	
    /** To get the string representation of this object. Overriding the
     * method in SOM_Declaration 
     * @return String The "name" of the facet corresponding to this
     * declaration 
     */
    public String toString() {
		return id.toString();
    }

    /** publishFunctionGeneration routine to print object module
     * Differs from publishSOM because the function is generated, not given
     * @param PrintWriter
     * @return void
     */
    public void publishFunctionGeneration(PrintWriter pw) {
	id.publishSOM(pw);
	if (observerList != null) {
	    observerList.publishSOM(pw);
	}
	pw.print(" :: ");
	recognizer.publishSOM(pw);
    }

    /* To get the desired object by traversal of SOM
     * to be implemented later
     */
    public ListEnumeration M__Terms() {
	return new ListEnumeration(null);
    }

    /* To get the string of the function generation
     * @param void
     * @return String
     */
    public String M__String(){
	try{
	    String str = new String();
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw,true);
	    publishFunctionGeneration(pw);
	    pw.flush();
	    pw.close();
	    str = sw.toString();
	    sw.close(); 
	    return str;
	}
	catch (Exception e) {
	    System.out.println(e);
	    return (String)null;
	}
    }
		
    /* To get the type of the function generation
     * @param void
     * @return String
     */
    public String M__Type(){
	return (String)null;
    }

    /* To get the type of the function generation
     * TO BE COMPLETED WHEN TYPE CHECKING IS FIXED
     * RETURN NULL FOR NOW
     * @param void
     * @return String
     */
    public Type M__Type(BaseObject parent, BaseObject
			newParent,Universe root, String identifier,
			Expression value) {
	Type type = null;
	return type;
    }

    /** The function that creates the ROM.Expression for the function
     * @param Factory,BaseObject,Universe
     * @return ROM.Expression(ROM.Facet)
     */
    public Expression M__Value(Factory romFactory,
			       BaseObject parentIn, Universe root){
	// I have only implemented function generation, the expression to this
	// function has NOT been implemented!!!
	// Will be null for now!!
	SOM_Expression expr = null;
	
	if (getObserverListing() != null){
	    Function function= romFactory.newFunction(parentIn);
	    ListEnumeration obsList = getObserverListing().observers();
	    while (obsList.hasMoreElements()){
		SOM_ObserverItem obsItem = (SOM_ObserverItem)
		    obsList.nextElement();
		function.addInterfaceItem(obsItem.buildROM
					      (romFactory, root, function));
	    }
	    if (expr != null)
		function.setExpression
		    (expr.M__Value(romFactory, function, root));
	    return (Expression)function;
	}
	
	else
	    return null;
    }
	
    /** The buildROM method for this high-level item
     * VOID BECAUSE IT IS GENERATION FUNCTIONS AND DIRECTLY ADDING TO ROOT!
     * @param ROMFactory, Factory object used to create objects in ROM
     * @return LabelledItem, corresponding to this facet  
     */
    public void buildROM(Factory romFactory, BaseObject parentIn, Universe
			 root) {
	LabelledItem declItem;
	declItem = romFactory.newLabelledItem(parentIn, null, null, null, M__String());
	Label declLabel =  romFactory.newLabel(declItem, id.toString());
	declItem.setLabel(declLabel);
	
	// Generation Constructor Functions
	Expression declValue = M__Value(romFactory, declItem, root);
	declItem.setValue(declValue);
	Type type = M__Type(parentIn, declItem, root, id.toString(),
			    declValue); 
	declItem.setType(type);

        // Added by Morel, July 14, 2001
	// Convert parent back to package or facet then manually add the generated fnc
	ROM.Facet packORfacetValue = (ROM.Facet)parentIn;
	packORfacetValue.addItem(declItem);
	
	// Generate Recognizer Functions
	if (getRecognizer() != null) {
	    getRecognizer().buildROM(romFactory, parentIn, root);
	}

	// Generation Observer Functions
	if (getObserverListing() != null){
	    ListEnumeration obsList = getObserverListing().observers();
	    while (obsList.hasMoreElements()){
		SOM_ObserverItem obsItem = (SOM_ObserverItem)
		    obsList.nextElement();
		obsItem.buildFncGenROM(romFactory, parentIn, root);
	    }
	}
    }
	
    /** publishSOM routine to print object model
     * @param PrintWriter pw
     * @return void
     */
    public void publishSOM(PrintWriter pw) {
	id.publishSOM(pw);
	if (observerList != null) {
	    observerList.publishSOM(pw);
	}
	pw.print(" :: ");
	recognizer.publishSOM(pw);
    }
    
}


