//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_DeclarationStatementZ.java,v 1.12 2001/03/26 19:04:58 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_DeclarationStatementZ.java,v $
//  Revision 1.12  2001/03/26 19:04:58  cvs
//  Completed the implementation for the change in export clauses. Also
//  added code in SOM_PackageDeclaration to take care of use clauses (case
//  was missing for when these clauses are found in declaration part of
//  packages).
//  -- Cindy
//
//  Revision 1.11  2001/03/12 23:54:37  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.10  2001/03/06 05:02:27  cvs
//  The M__Value method has been implemented for SelectedName and some
//  errors in the Meta functions have been removed in the other files 
//  --srinivas
//
//  Revision 1.9  2000/12/03 23:55:26  cvs
//  Added the M__Value method to the Literal classes.Previously they
//  were returning null.The M__Type method for all the Declaration
//  statement classes has been completed 
//  --srinivas
//
//  Revision 1.8  2000/07/11 00:11:09  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are 
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.7  2000/06/22 17:22:40  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.6  2000/05/19 00:03:59  cvs
//  restoring the files after the file system had crashed
//  --srinivas
//
//  Revision 1.6  2000/05/06 00:14:53  cvs
//   The Meta Functions for the term items have been changed to the
//   class SOM_LabelledExpression .The M__Value and M__String have
//   been implemented for the AssociationListing.The meta functions
//   for the declaration items are also implemented in the
//   SOM_DeclarationStatement 
//  --srinivas
//
//  Revision 1.5  2000/04/07 00:16:57  cvs
//  Updated a method which was giving conflicts due to error in naming.
//  - Krishna
//
//  Revision 1.4  2000/04/06 19:13:19  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.3  2000/04/05 20:29:28  cvs
//  There are changes in the  SOM_DeclarationStatementZ because of the
//  addition of some accessor function 
//  --srinivas
//
//  Revision 1.2  2000/03/07 22:58:15  cvs
//  Modidfied the class due the change in syntax for function, facet
//  declaration statements and the unnamed function 
//  --srinivas
//
//  Revision 1.1  2000/02/10 03:56:28  cvs
//  Modified the grammar for incorporating records, tuples and "let"
//  expressions. Also populated the object model with these above
//  constructs, and completed the pretty printing for them. 
//  - krishna
//
//
//
//=============================================================================
package SOM;
import util.ListEnumeration;
import java.io.PrintWriter;
import ROM.*;

/** This represents the <B>DeclarationStatementZ</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 */
public class SOM_DeclarationStatementZ
extends SOM_BaseClass {

	private SOM_Expression typeExpr;
	private SOM_Expression valueExpr = null;

    //	private SOM_ConstantDeclarationStatement cstDeclStmt= null;
    //	private SOM_FunctionDeclarationStatement funcDeclStmt= null;

	/** The primary constructor
	 * @return Not applicable
	 */
	public SOM_DeclarationStatementZ() {
		super();
	}

	/** To set the type expression
	 * @param expr The SOM_Expression to be set
	 * @return void
	 */
	public void setTypeExpression(SOM_Expression expr) {
		typeExpr = expr;
	}			 
	 
	/** To set the value expression
	 * @param value The SOM_Expression object to be set
	 * @return void
	 */
	public void setValueExpression(SOM_Expression value) {
		valueExpr = value;
	}

	/** To get the type expression
	 * @return The SOM_Expression object for this declaration
	 */
	public SOM_Expression getTypeExpression() {
		return typeExpr;
	}

	/** To get the value expression
	 * @return The SOM_Expression object representing the value for
	 * this declaration 
	 */
	public SOM_Expression getValueExpression() {
		return valueExpr;
	}

	//Added by srinivas on Nov 29th, 2000
	/** The function is to get the Type of the set declaration
	 * @param parent
	 * @param root The Universe object to search for predefined types
	 * @param value The Expression object to add to bunchValue of type
	 * @return ROM.Type
	 */
	public Type M__Type(BaseObject parent, Universe root, Expression value){
		SOM_Expression exp = getTypeExpression();
		Type type = null;
		if (exp != null){
			SOM_TypeDeclaration tp = exp.getResolvedType();
			if (tp != null){
				String str = tp.toString();
				type = root.getType(str);
				if (type == null){
					type = ((Facet)parent).getType(str);
				}
			}
		}
		/*
		  if (type instanceof UserDefinedType)
		  if (value != null)
		  ((UserDefinedType)type).addElement(value);
		*/
		return type;
	}
	
	/** publishSOM routine to print object module
     * @param PrintWriter
     * @return void - mak
     */
    public void publishSOM(PrintWriter pw){
		typeExpr.publishSOM(pw);
		if (valueExpr != null) {
			pw.print(" IS ");
			valueExpr.publishSOM(pw);
		}
    }				
				
}
