package SOM;

import util.ListEnumeration;
import java.util.Hashtable;
import java.util.Enumeration;
import SYM.SYM_TypeInconsistencyException;

/** This represents the <B>EnumerationItemType</B> object
 * @author <I>Brandon Morel</I> (morel@ittc.ku.edu)
 */
public class SOM_EnumerationItemType
extends SOM_TypeDeclaration {

	// The fields in the EnumerationItem
	// The id of the field is the key and the value associated with
	// the key, the type.
	Hashtable fields = new Hashtable();

	/** The primary constructor
	 * @param myType The SOM_TypeDeclaration corresponding to the
	 * "bunch MetaType"
	 * @param myId The Identifier containing the name of the type
	 * @return Not applicable
	 */
	public SOM_EnumerationItemType(SOM_Identifier myId, SOM_TypeDeclaration myType) {
		super(myId, myType);
	}

	/** Another constructor
	 * @param myType The SOM_TypeDeclaration corresponding to the
	 * "bunch MetaType"
	 * @return Not applicable
	 */
	public SOM_EnumerationItemType(SOM_TypeDeclaration myType) {
		// The default name is setType
		this(new SOM_Identifier("EnumerationItemType"), myType);
	}

	/** To set the members and their type
	 * @param idIn The SOM_Identifier for the name of the field
	 * @param memberTypeIn The SOM_TypeDeclaration corresponding to field
	 * @return void
	 */
	public void setFieldType(SOM_Identifier idIn, 
							 SOM_TypeDeclaration memberTypeIn) {
		fields.put(idIn, memberTypeIn);
	}

	/** To get the member type associated with a field
	 * @param idIn The SOM_Identifier for the name of the field
	 * @return The SOM_TypeDeclaration of member
	 */
	public SOM_TypeDeclaration getMemberType(SOM_Identifier idIn) {
		return (SOM_TypeDeclaration)fields.get(idIn);
	}

	/** To get the fields
	 * @return The Hashtable containing the fields
	 */
	public Hashtable getFields() {
		return fields;
	}

	/** To implement a version of isDistinguishable for EnumerationItem types
	 * @param typeOne The first SOM_EnumerationItemType
	 * @param typeTwo The next SOM_EnumerationItemType to be distinguished
	 * against typeOne
	 * @return true if they are distinguishable, false otherwise
	 */
	public static boolean isDistinguishable(SOM_EnumerationItemType typeOne,
											SOM_EnumerationItemType typeTwo) { 
		Hashtable fieldsOne = typeOne.getFields();
		Hashtable fieldsTwo = typeTwo.getFields();
		Enumeration keysOne = fieldsOne.keys();
		Object key;
		SOM_TypeDeclaration fieldTypeOne;
		SOM_TypeDeclaration fieldTypeTwo;

		if (fieldsOne.equals(fieldsTwo))
			return false;

		if (fieldsOne.size() != fieldsTwo.size())
			return true;
			
		while (keysOne.hasMoreElements()) {
			key = keysOne.nextElement();
			fieldTypeOne = (SOM_TypeDeclaration)fieldsOne.get(key);
			fieldTypeTwo = (SOM_TypeDeclaration)fieldsTwo.get(key);
			if (SOM_TypeDeclaration.isDistinguishable
				(fieldTypeOne, fieldTypeTwo))
				return true;
		}
		
		return false;
	}

	/** To ascertain "compatiblity" for bindings of the kind lhs <- rhs
	 * @param typeLHS The EnumerationItemType corresponding to the lhs object
	 * @param typeRHS The EnumerationItemType corresponding to the rhs object
	 * @return true, if the rhs is assignable to the lhs, false otherwise
	 */
	public static boolean
			isLeftTypeCompatibleWithRightType(SOM_EnumerationItemType typeLHS,
											  SOM_EnumerationItemType typeRHS) { 
		Hashtable fieldsLHS = typeLHS.getFields();
		Hashtable fieldsRHS = typeRHS.getFields();
		Enumeration keysLHS = fieldsLHS.keys();
		Enumeration keysRHS;
		Object keyLHS;
		Object keyRHS = null;		
		SOM_TypeDeclaration fieldTypeLHS;
		SOM_TypeDeclaration fieldTypeRHS;
		String keyId;

		if (fieldsRHS.equals(fieldsLHS))
			return true;

		if (fieldsRHS.size() != fieldsLHS.size())
			return false;
			
		while (keysLHS.hasMoreElements()) {
			keyLHS = keysLHS.nextElement();
			keyId = keyLHS.toString();
			
			keysRHS = fieldsRHS.keys();
			while (keysRHS.hasMoreElements()) {
				keyRHS = keysRHS.nextElement();
				if (keyRHS.toString().equals(keyId)) {
					break;
				}
			}
			if (!keyRHS.toString().equals(keyId)) {
				throw new SYM.SYM_TypeInconsistencyException
					("Field with key: " + keyId + " missing in RHS");
			}
			fieldTypeRHS = (SOM_TypeDeclaration)fieldsRHS.get(keyRHS);
			fieldTypeLHS = (SOM_TypeDeclaration)fieldsLHS.get(keyLHS);
			if (fieldTypeLHS == null) { 
				throw new NullPointerException("No type corresponding" +
											   " to field name: " + 
											   keyId);
			}
			if (fieldTypeRHS == null) { 
				throw new NullPointerException("No type corresponding" +
											   " to field name: " + 
											   keyId);
			}
			if (!fieldTypeRHS.isPredicateSubtypeOf(fieldTypeRHS))
				return false;
		}
		
		return true;
	}
}


