package SOM;

import util.ListEnumeration;
import java.util.Hashtable;
import java.util.Enumeration;
import SYM.SYM_TypeInconsistencyException;
import SYM.SYM_IllegalRedeclarationException;

/** This represents the <B>EnumerationType</B> object
 * @author <I>Brandon Morel</I> (morel@ittc.ku.edu)
 */
public class SOM_EnumerationType
    extends SOM_TypeDeclaration {

    // The fields in the enumeration
    // The id of the field is the key and the value associated with
    // the key, the type.
    Hashtable fields = new Hashtable();

    // This will be the type of the members of the set
    protected SOM_TypeDeclaration memberType;

    /** The primary constructor
     * @param myType The SOM_TypeDeclaration corresponding to the
     * "bunch MetaType"
     * @param myId The Identifier containing the name of the type
     * @return Not applicable
     */
    public SOM_EnumerationType(SOM_Identifier myId, SOM_TypeDeclaration myType) {
	super(myId, myType);
    }

    /** Another constructor
     * @param myType The SOM_TypeDeclaration corresponding to the
     * "bunch MetaType"
     * @return Not applicable
     */
    public SOM_EnumerationType(SOM_TypeDeclaration myType) {
	// The default name is setType
	this(new SOM_Identifier("enumerationType"), myType);
    }

    /** To set the type of members
     * @param memberTypeIn The SOM_TypeDeclaration corresponding to members
     * @return void
     */
    public void setMemberType(SOM_TypeDeclaration memberTypeIn) {
        memberType = memberTypeIn;
    }
     
    /** To get the member type
     * @return The SOM_TypeDeclaration of member
     */
    public SOM_TypeDeclaration getMemberType() {
        return memberType;
    }

    /** To set the members and their type
     * @param enumIn The SOM_ENumerationItem for the name of the field
     * @param memberTypeIn The SOM_TypeDeclaration corresponding to field (always univ)
     * @return void
     */
    public void setFieldType(SOM_EnumerationItem enumIn,
                             SOM_TypeDeclaration memberTypeIn) {
	
        Object list = fields.get(enumIn.toString());

        if (list == null)
            fields.put(enumIn.toString(), memberTypeIn);
	
        else
            throw new SYM_IllegalRedeclarationException("SOM_EnumerationType:  Illegal "
                                                        + "redeclaration of value "
                                                        + enumIn.toString());
    }


    /** To get the member type associated with a field
     * @param enumerationIn The SOM_EnumerationItem for the name of the field
     * @return The SOM_TypeDeclaration of member
     */
    // Not really needed because the member type is always univ
    public SOM_TypeDeclaration getMemberType(SOM_EnumerationItem enumIn) {
	return (SOM_TypeDeclaration)fields.get(enumIn.toString());
    }

    /** To get the fields
     * @return The Hashtable containing the fields
     */
    public Hashtable getFields() {
	return fields;
    }
    
    /** To implement a version of isDistinguishable for enumeration types
     * @param typeOne The first SOM_EnumerationType
     * @param typeTwo The next SOM_EnumerationType to be distinguished
     * against typeOne
     * @return true if they are distinguishable, false otherwise
     */
    public static boolean isDistinguishable(SOM_EnumerationType typeOne,
					    SOM_EnumerationType typeTwo) { 
	Hashtable fieldsOne = typeOne.getFields();
	Hashtable fieldsTwo = typeTwo.getFields();
	Enumeration keysOne = fieldsOne.keys();
	Object key;
	SOM_TypeDeclaration fieldTypeOne;
	SOM_TypeDeclaration fieldTypeTwo;

	if (fieldsOne.equals(fieldsTwo))
	    return false;

	if (fieldsOne.size() != fieldsTwo.size())
	    return true;
			
	while (keysOne.hasMoreElements()) {
	    key = keysOne.nextElement();
	    fieldTypeOne = (SOM_TypeDeclaration)fieldsOne.get(key);
	    fieldTypeTwo = (SOM_TypeDeclaration)fieldsTwo.get(key);
	    if (SOM_TypeDeclaration.isDistinguishable
		(fieldTypeOne, fieldTypeTwo))
		return true;
	}
		
	return false;
    }

    /** To ascertain "compatiblity" for bindings of the kind lhs <- rhs
     * @param typeLHS The EnumerationType corresponding to the lhs object
     * @param typeRHS The EnumerationType corresponding to the rhs object
     * @return true, if the rhs is assignable to the lhs, false otherwise
     */
    public static boolean
	isLeftTypeCompatibleWithRightType(SOM_EnumerationType typeLHS,
					  SOM_EnumerationType typeRHS) { 
	Hashtable fieldsLHS = typeLHS.getFields();
	Hashtable fieldsRHS = typeRHS.getFields();
	Enumeration keysLHS = fieldsLHS.keys();
	Enumeration keysRHS;
	Object keyLHS;
	Object keyRHS = null;		
	SOM_TypeDeclaration fieldTypeLHS;
	SOM_TypeDeclaration fieldTypeRHS;
	String keyId;

	if (fieldsRHS.equals(fieldsLHS))
	    return true;

	if (fieldsRHS.size() != fieldsLHS.size())
	    return false;
			
	while (keysLHS.hasMoreElements()) {
	    keyLHS = keysLHS.nextElement();
	    keyId = keyLHS.toString();
			
	    keysRHS = fieldsRHS.keys();
	    while (keysRHS.hasMoreElements()) {
		keyRHS = keysRHS.nextElement();
		if (keyRHS.toString().equals(keyId)) {
		    break;
		}
	    }
	    if (!keyRHS.toString().equals(keyId)) {
		throw new SYM.SYM_TypeInconsistencyException
		    ("Field with key: " + keyId + " missing in RHS");
	    }
	    fieldTypeRHS = (SOM_TypeDeclaration)fieldsRHS.get(keyRHS);
	    fieldTypeLHS = (SOM_TypeDeclaration)fieldsLHS.get(keyLHS);
	    if (fieldTypeLHS == null) { 
		throw new NullPointerException("No type corresponding" +
					       " to field name: " + 
					       keyId);
	    }
	    if (fieldTypeRHS == null) { 
		throw new NullPointerException("No type corresponding" +
					       " to field name: " + 
					       keyId);
	    }
	    if (!fieldTypeRHS.isPredicateSubtypeOf(fieldTypeRHS))
		return false;
	}
		
	return true;
    }
}


