//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_FacetAndExpression.java,v 1.4 2001/03/12 23:54:38 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_FacetAndExpression.java,v $
//  Revision 1.4  2001/03/12 23:54:38  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.3  2000/12/05 03:05:03  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.2  2000/08/01 22:48:48  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.1  2000/02/17 01:27:04  cvs
//  Implemented the facet expression grammar to incorporate facet composition.
//  Also updated the "use" clause with the above changes.
//  - krishna.
//
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;
import java.io.PrintWriter;
import ROM.*;

/** This represents the <B>FacetAndExpression</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 */
public class SOM_FacetAndExpression
extends SOM_ExpressionWithDyadicOperator {
	// This represents the <FacetAndExpression> object, for which the
	// grammar is 
	// <facetAndExpression> ::== <facetSelectedName>
	// (<facetAndExpressionTail>)* 
	//
	// <facetAndExpressionTail> ::== {AND} <facetSelectedName>

	//private SOM_FacetSelectedName selName;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_FacetAndExpression() {
		super();
	}

	/** To set the facetSelectedName
	 * @param selNameIn The SOM_FacetSelectedName object to be set
	 * @return void
	 */
	public void setFacetSelectedName(SOM_FacetSelectedName selNameIn) {
		//selName = selNameIn;
		setStrongerExpression(selNameIn);
	}

	/** To get the facetSelectedName
	 * @return SOM_FacetSelectedName
	 */
	public SOM_FacetSelectedName getFacetSelectedName() {
		//return (SOM_FacetSelectedName)selName;
		return (SOM_FacetSelectedName)getExpression();
	}

	/** To add to the facetAndExprTailList
	 * @param facetAndExprTail The SOM_FacetAndExpressionTail object
	 * to be added 
	 * @return void
	 */
	public void addToFacetAndExpressionTailList
		(SOM_FacetAndExpressionTail facetAndExprTail) {
		addToAbstractExpressionTailList(facetAndExprTail);
	}

	/** To retrieve the listEnumeration of the facetAndExprTailList
	 * @return The ListEnumeration corresponding to the facetAndExprTailList
	 */
	public ListEnumeration expressions() {
		return getExpressionTailList().elements();
	}

	//added by ckong March 5, 2001
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
     *@param Universe root
	 * @return Expression
     */
	public Expression M__Value(Factory romFactory, BaseObject parent,
							   Universe root){
		Expression leftNode;
		ListEnumeration expressionTailEnum = expressions();
		if (expressionTailEnum != null){
			if(expressionTailEnum.hasMoreElements()){
				Formula intermExpr = new Formula(parent);
				leftNode = getFacetSelectedName().
					M__Value(romFactory, intermExpr, root);
				Formula retExpr = intermExpr;
				while (expressionTailEnum.hasMoreElements()) {
					SOM_FacetAndExpressionTail exprTail = 
					(SOM_FacetAndExpressionTail)expressionTailEnum.
						nextElement();
					SOM_DyadicOperator oper = exprTail.getOperator();
					Expression rightNode =
						exprTail.getFacetSelectedName().
						M__Value(romFactory, intermExpr, root);
					intermExpr.setLeftOp(leftNode);
					String str = oper.toString();
					Type operType = 
						root.getType(oper.getResolvedType().toString());
					OperatorReference operRef = new
						OperatorReference(intermExpr,
										  root.labelLookup(str), operType);
					intermExpr.setOperator(operRef);
					intermExpr.setRightOp(rightNode);
					retExpr = intermExpr;
					leftNode = intermExpr;
					intermExpr = new Formula(parent);
				}
				return retExpr;
			}else {
				leftNode = getFacetSelectedName().
					M__Value(romFactory,parent,root);
				return leftNode;
			}
		}else {
			leftNode = getFacetSelectedName().
				M__Value(romFactory,parent,root);
			return leftNode;
		}
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		ListEnumeration expressionTailEnum = this.expressions();
		SOM_FacetAndExpressionTail facetAndExprTail;

		//selName.publishSOM(pw);
		getFacetSelectedName().publishSOM(pw);

		while (expressionTailEnum.hasMoreElements()) {
			facetAndExprTail = 
				(SOM_FacetAndExpressionTail)expressionTailEnum.nextElement();
			facetAndExprTail.publishSOM(pw);
		}
	}

	/** The publishPVS routine for this class
	 * @return void
     * To be implemented as needed */
	public void publishPVS(PrintWriter pw) {
        publishSOM(pw);
	}
}
