//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_ImpliesExpression.java,v 1.14 2001/03/12 23:54:39 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_ImpliesExpression.java,v $
//  Revision 1.14  2001/03/12 23:54:39  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.13  2000/12/06 01:18:16  cvs
//  Modified the loader for expressions to allow expressions such as:
//  var = var1 op var2 op var3 (without the use of parentheses to group
//  items)
//  -- Cindy
//
//  Revision 1.12  2000/11/27 23:49:56  cvs
//  Previously was not taking the tick and at operators into
//  consideration while building the ROM so made changes to include
//  these operators. 
//  --srinivas
//
//  Revision 1.11  2000/11/18 00:18:27  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.10  2000/11/02 23:11:54  cvs
//  Some changes to the M__Value method so that it returns just an
//  Expression and not a Formula only 
//  --srinivas
//
//  Revision 1.9  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the
//  terms(SOM_LabelledExpression) and it returns a ROM Expression. 
//  --srinivas
//
//  Revision 1.8  2000/08/01 22:48:50  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.7  2000/06/22 17:22:43  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.6  2000/04/06 19:13:22  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.5  2000/03/23 00:22:48  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.4  2000/02/10 00:37:21  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.3  2000/02/08 18:42:31  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help 
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.2  1999/11/05 23:47:05  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:38  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:50:23  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.3  1999/07/14 23:23:50  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.2  1999/06/30 22:20:00  rkamath
//  Compeleted the object model coding for the expression grammar
//  The expression objects now inherit from a AbstractExpression class
//
//  Revision 1.1  1999/06/30 02:19:03  rkamath
//  Added a lot more files dealing with the expression OM.
//  Also modified some previous classes
//  Now the ROOT in the hierarchy is the SOM_Unifier interface.
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;

/** This represents the <B>ImpliesExpression</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_ImpliesExpression
extends SOM_ExpressionWithDyadicOperator {
	// This represents the <impliesExpression> object, for which the grammar is
	// <impliesExpression> ::==
	//   <andExpression> (<impliesExpressionTail>)*
	//
	// <impliesExpressionTail> ::== IMPLIES <andExpression>
	
	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_ImpliesExpression() {
		super();
	}

	/** To set the andExpression
	 * @param expressionIn The SOM_AndExpression object to be set
	 * @return void
	 */
	public void setAndExpression(SOM_AndExpression expressionIn) {
		setStrongerExpression(expressionIn);
	}

	 /** To get the andExpression
		* @return The SOM_AndExpression object for this expression
		*/
	 public SOM_AndExpression getAndExpression() {
			 return (SOM_AndExpression)getExpression();
	 }

	/** To add to the impliesTailList
	 * @param eTail The SOM_ImpliesExpressionTail object to be added
	 * @return void
	 */
	public void addToImpliesExpressionTailList
		(SOM_ImpliesExpressionTail eTail)
	{
		addToAbstractExpressionTailList(eTail);
	}

	/** To retrieve the listEnumeration of the expressionTailList
	 * @return The ListEnumeration corresponding to the expressionTailList
	 */
	public ListEnumeration expressions() {
		return getExpressionTailList().elements();
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		ListEnumeration expressionTailEnum = this.expressions();
		SOM_ImpliesExpressionTail impliesExprTail;

		getExpression().publishSOM(pw);

		while (expressionTailEnum.hasMoreElements()) {
			impliesExprTail =
				(SOM_ImpliesExpressionTail)expressionTailEnum.nextElement(); 
			impliesExprTail.publishSOM(pw);
		}
	}

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root){

		ListEnumeration expressionTailEnum = expressions();
		Expression leftNode;
		if (expressionTailEnum !=null){
			if (expressionTailEnum.hasMoreElements()){
				Formula  intermExpr = new Formula(parent);
				leftNode = getAndExpression().M__Value
					(romFactory, intermExpr, root);
				Formula retExpr = intermExpr;
				while (expressionTailEnum.hasMoreElements()) {
					SOM_ImpliesExpressionTail impliesExprTail =
						(SOM_ImpliesExpressionTail)expressionTailEnum.
						nextElement();
					SOM_DyadicOperator oper = impliesExprTail.
						getImpliesOperator();
					Expression rightNode =
						impliesExprTail.getAndExpression().
						M__Value(romFactory, intermExpr, root);
					intermExpr.setLeftOp(leftNode);
					String str = oper.toString();
					if (str.equals("implies"))
						str = "=>";
					Type operType = 
						root.getType(oper.getResolvedType().toString());
					OperatorReference operRef = new
						OperatorReference(intermExpr,
										  root.labelLookup(str), operType);
					intermExpr.setOperator(operRef);
					intermExpr.setRightOp(rightNode);
					//leftNode = new Formula(retExpr);
					retExpr = intermExpr;
					leftNode = intermExpr;
					intermExpr = new Formula(parent);
				}
				return retExpr;
			}else{
				leftNode = getAndExpression().M__Value(romFactory,parent,root);
				return leftNode;
			}
		}else{
			leftNode = getAndExpression().M__Value(romFactory,parent,root);
			return leftNode;
		}
	}
		
		/** The publishPVS routine for this class
		 * @return void
		 */
		public void publishPVS(PrintWriter pw) {
				ListEnumeration expressionTailEnum = this.expressions();
				SOM_ImpliesExpressionTail impliesExprTail;

				getExpression().publishPVS(pw);

				while (expressionTailEnum.hasMoreElements()) {
						impliesExprTail =
								(SOM_ImpliesExpressionTail)expressionTailEnum.nextElement(); 
						impliesExprTail.publishPVS(pw);
				}
		}
}
