//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_InterfaceListing.java,v 1.7 2000/06/22 17:22:43 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_InterfaceListing.java,v $
//  Revision 1.7  2000/06/22 17:22:43  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.6  2000/05/19 00:03:59  cvs
//  restoring the files after the file system had crashed
//  --srinivas
//
//  Revision 1.6  2000/05/10 16:22:37  cvs
//  used the typeName to extarct the type information of components needed in SOCCER - makarand
//
//  Revision 1.5  2000/04/02 18:25:55  cvs
//  modified publishPVS functions to return 'in' and 'out' variables - by makarand for soccer
//
//  Revision 1.4  2000/03/23 19:16:39  cvs
//  added publishPVS routines for SOCCER - makarand
//
//  Revision 1.3  2000/02/10 00:37:22  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  2000/02/08 18:42:34  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes (with the help
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the object model.
//  -- Cindy.
//
//  Revision 1.1  1999/10/27 20:43:40  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:50:34  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.2  1999/07/22 22:53:22  rkamath
//  Lot of changes especially in terms of adding accessors to the OM
//  This does not compile completely, the toString() of SOM_FormalIdentifier
//  is to be completed. This commit is purely for records
//
//  Revision 1.1  1999/06/30 22:20:06  rkamath
//  Compeleted the object model coding for the expression grammar
//  The expression objects now inherit from a AbstractExpression class
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

import java.io.PrintWriter;

/** This represents the <B>InterfaceListing</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_InterfaceListing
extends SOM_BaseClass{

	private SOM_InterfaceItemList interfaceList = new SOM_InterfaceItemList();

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_InterfaceListing() {
		super();
	}

	/** To add to the interfaceItemList
	 * @param item The SOM_InterfaceItem to be added
	 * @return void
	 */
	public void addToInterfaceItemList(SOM_InterfaceItem item) {
		interfaceList.addElement(item);
	}

	/** To retrieve the listEnumeration of the interfaceItemList
	 * @return The ListEnumeration corresponding to the interfaceItemList
	 */
	public ListEnumeration interfaces() {
		return interfaceList.elements();
	}

	/** To get the size of the list associated with the listing
	 * @return The size of the list
	 */
	public int size() {
		return interfaceList.size();
	}

	/** The publishSOM routine for this class
     * @return void
     */
    public void publishSOM(PrintWriter pw) {
		ListEnumeration interfaceItemEnum = interfaces();
		SOM_InterfaceItem interfaceItem;

		pw.print("( ");
		((SOM_InterfaceItem)interfaceItemEnum.nextElement()).publishSOM(pw);
		while (interfaceItemEnum.hasMoreElements()) {
			pw.print("; ");
			interfaceItem = (SOM_InterfaceItem) interfaceItemEnum.nextElement(); 
			interfaceItem.publishSOM(pw);
		}
		pw.print(") ");
    }	

		/** added by makarand Mar 8 2000
		 *  PublishPVS routine used for SOCCER
		 *  @return void
		 */
		
		public void publishPVS(PrintWriter pw,SOM_List prelist, SOM_List inplist, SOM_List outlist) {
				try{
						ListEnumeration interfaceItemEnum = interfaces();
						SOM_InterfaceItem interfaceItem;
						pw.println();
						((SOM_InterfaceItem)interfaceItemEnum.nextElement()).publishPVS(pw,prelist,inplist,outlist);
						while (interfaceItemEnum.hasMoreElements()) {
								pw.println("");
								interfaceItem = (SOM_InterfaceItem)interfaceItemEnum.nextElement();
								interfaceItem.publishPVS(pw,prelist,inplist,outlist);
						}
						pw.println();
				}
				catch(Exception e)
						{
								System.out.println("Error in SOM_InterfaceListing.publishPVS"+  e);
						}
				
		}
}

