//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_LabelDeclaration.java,v 1.4 2000/06/22 17:22:44 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_LabelDeclaration.java,v $
//  Revision 1.4  2000/06/22 17:22:44  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.3  2000/02/10 00:37:23  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  1999/11/13 20:01:27  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/10/27 20:43:41  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:50:44  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.4  1999/10/01 15:23:30  rkamath
//  Commit for record - more cleaner typing ideas.
//
//  Revision 1.3  1999/08/01 20:54:32  rkamath
//  Made a lot of changes with regard to types and checking. I'm convinced that
//  there are going to me hundreds more changes until I stabilize on how to
//  import and [de]serialize
//
//  Revision 1.2  1999/07/28 01:59:41  rkamath
//  Changed all the SOM_Declaration references to SYM_SymboltableEntry as far
//  as lookUp and typeResolution are concerned
//
//  Revision 1.1  1999/07/16 21:50:31  rkamath
//  Changed everyexpression to point to the type declaration instead of having
//  a private variable which would indicate the resulting type
//
//
//=============================================================================
package SOM;

import SYM.SYM_Symboltable;

/** This represents the <B>LabelDeclaration</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_LabelDeclaration
extends SOM_Declaration {

	private SOM_Label label;

	// itemDecl is null when the label is not referencing a facet. In
	// the future, it should never be null as a label always
	// references and item
	private SOM_Declaration itemDecl = null;

	/** The primary constructor
	 * @param labelIn The SOM_Label corresponding to the declaration
	 * of the label
	 * @param typeIn The SOM_TypeDeclaration corresponding to the type
	 * @param itemDeclIn The SOM_Declaration associated with the item
	 * that is being renamed by the label  
	 * @return Not applicable
	 */
	public SOM_LabelDeclaration(SOM_Label labelIn, 
								SOM_TypeDeclaration typeIn,
								SOM_Declaration itemDeclIn) 
	{
		super();
		label = labelIn;
		setKind(SOM_Declaration.LABEL);
		setType(typeIn);
		itemDecl = itemDeclIn;
	}

	/** The primary constructor
	 * @param labelIn The SOM_Label corresponding to the declaration
	 * of the label
	 * @param typeIn The SOM_TypeDeclaration corresponding to the type
	 * @return Not applicable
	 */
	public SOM_LabelDeclaration(SOM_Label labelIn, 
								SOM_TypeDeclaration typeIn)
	{
		super();
		label = labelIn;
		setKind(SOM_Declaration.LABEL);
		setType(typeIn);
	}

	/** To get the string representation of this object. Overrides the
	 * method in SOM_Declaration
	 * @return String representation of this object - name of the label
	 */
	public String toString() {
		return label.getIdentifier().toString();
	}

	/** To get the Symboltable associated with the item.
	 * Overrides the method in Declaration as any symbol associated
	 * with a label is that of the item being renamed
	 * @return The SYM_Symboltable associated with the item renamed by
	 * this label
	 */
	public SYM_Symboltable getMySymboltable() {
		System.out.println("Getting symboltable for label: " + toString());
		if (itemDecl != null)
			return itemDecl.getMySymboltable();
		else
			return null;
	}

}
