//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_LetDeclarationStatement.java,v 1.3 2000/06/22 17:22:44 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_LetDeclarationStatement.java,v $
//  Revision 1.3  2000/06/22 17:22:44  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.2  2000/04/06 19:13:22  cvs
//  Type checking is done for declaration of variables (simple types), for declaration of new types and for use of basic operators.
//   -- Cindy
//
//  Revision 1.1  2000/02/10 03:56:28  cvs
//  Modified the grammar for incorporating records, tuples and "let" expressions. Also populated the object model with these above constructs, and completed the pretty printing for them.
//  - krishna
//
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

import java.io.PrintWriter;

/** This represents the <B>LetDeclarationStatement</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 */
public class SOM_LetDeclarationStatement
extends SOM_Declaration {

	private SOM_Identifier id;
	private SOM_Expression typeExpr, valueExpr;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_LetDeclarationStatement() {
		super();
	}

	/** To set the identifier
	 * @param idIn the SOM_Identifier to be set
	 * @return void
	 */
	public void setIdentifier(SOM_Identifier idIn) {
		id = idIn;
	}

	/** To get the identifier
	 * @param null
	 * @return SOM_Identifier
	 */
	public SOM_Identifier getIdentifier() {
		return id;
	}

	/** To set the typeExpr
	 * @param typeExprIn the SOM_Expression to be set
	 * @return void
	 */
	public void setTypeExpression(SOM_Expression typeExprIn) {
		typeExpr = typeExprIn;
	}

	/** To get the typeExpr
	 * @return SOM_Expression
	 */
	public SOM_Expression getTypeExpression() {
		return typeExpr;
	}
	
	/** To set the valueExpr
	 * @param valueExprIn the SOM_Expression to be set
	 * @return void
	 */
	public void setValueExpression(SOM_Expression valueExprIn) {
		valueExpr = valueExprIn;
	}

	/** To get the valueExpr
	 * @return SOM_Expression
	 */
	public SOM_Expression getValueExpression() {
		return valueExpr;
	}
	
	/** To get the name of the type of the declaration
	 * @return The SOM_Name for the Type
	 */
	public SOM_TypeDeclaration getExpressionType() {
		typeExpr.resolveFinalType();
		return	typeExpr.getResolvedType();
	}

	/** publishSOM routine to print the object module
     * @param PrintWriter
     * @return void
     */
    public void publishSOM(PrintWriter pw){
		id.publishSOM(pw);
		pw.print(" :: ");
		typeExpr.publishSOM(pw);
		pw.print(" IS ");
		valueExpr.publishSOM(pw);
    }
}

