//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_Literal.java,v 1.16 2001/07/03 14:53:58 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_Literal.java,v $
//  Revision 1.16  2001/07/03 14:53:58  cvs
//
//  Initial implementation of enumerations.
//  --Morel
//
//  Revision 1.15  2000/11/18 00:18:28  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.14  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the terms(SOM_LabelledExpression) and it returns a ROM Expression.
//  --srinivas
//
//  Revision 1.13  2000/08/01 22:48:52  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.12  2000/07/11 00:11:12  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and sequence are
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.11  2000/06/22 17:22:45  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.10  2000/04/28 19:34:08  cvs
//  Added some more of typechecking. Added parsing of the $ operator
//  and constant literal. 
//  -- Cindy
//
//  Revision 1.9  2000/04/06 19:13:23  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.8  2000/03/23 00:22:48  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.7  2000/02/10 00:37:25  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.6  2000/02/08 18:42:35  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary classes (due to changes brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.5  1999/11/17 18:31:55  cvs
//  Coded resolution of final type (unfolding overload resolution) and included
//  actions for TCC generation
//
//  Revision 1.4  1999/11/15 22:59:16  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.3  1999/11/15 03:16:28  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.2  1999/11/13 20:01:28  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/10/27 20:43:43  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:50:54  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.7  1999/10/01 15:23:31  rkamath
//  Commit for record - more cleaner typing ideas.
//
//  Revision 1.6  1999/07/28 01:59:42  rkamath
//  Changed all the SOM_Declaration references to SYM_SymboltableEntry as far
//  as lookUp and typeResolution are concerned
//
//  Revision 1.5  1999/07/21 00:28:14  rkamath
//  Small changes and consistency corrections...
//
//  Revision 1.4  1999/07/13 14:27:31  rkamath
//  Changed the publishRosetta() function to take in a PrintWriter
//  Added some new classes too.
//  Also commented out currency based operations from the SOM_List class. Hope-
//  fully at a later date I'll include them in the SOM_ListEnumeration class if
//  found necessary.
//
//  Revision 1.3  1999/07/07 21:22:20  rkamath
//  Added some literal support
//
//  Revision 1.2  1999/07/06 21:01:42  rkamath
//  Having added the type expression syntax the corresponding OM...etc
//  Also now, only subclasses of RuntimeException are being thrown
//
//  Revision 1.1  1999/06/27 17:33:34  rkamath
//  Added some of the object model for the Expression grammar and a few changes to
//  the other files..
//
//
//=============================================================================
package SOM;

import SYM.SYM_UnresolvableException;

import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;

/** This represents the <B>Literal</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
// Added implement class SOM_EnumerationItem
// Morel, June 30, 2001
public abstract class SOM_Literal
extends SOM_AbstractExpression
implements SOM_Factor, SOM_Terminal, SOM_EnumerationItem {

	protected String literal;

	// This is saved purely for information purposes - it is also used in
	// getNames function
	SOM_DeclarationList declList;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_Literal() {
		super();
	}

	/** The second constructor
	 * @param literalIn The String representation of the literal
	 * @return Not applicable
	 */
	public SOM_Literal(String literalIn) {
		super();
		literal = literalIn;
	}

	/** To set the literal
	 * @param literalIn The String representation of the literal
	 * @return void
	 */
	public void setLiteral(String literalIn) {
		literal = literalIn;
	}

	/** To get the literal
	 * @return The String representation of the literal
	 */
	public String getLiteral() {
		return literal;
	}

	/** To set the declarations. This also populates the TypeList
	 * setTypeList to point to the possible types
   * I wish we had multiple inheritance - I wouldn't have to duplicate
   *  methods here and in SOM_Name
   * @param declsIn The SOM_DeclarationList to be set
   * @return void
   */
  public void setDeclarations(SOM_DeclarationList declsIn) {
		SOM_TreeNodeList toReturn = new SOM_TreeNodeList();
		ListEnumeration enum = declsIn.elements();
		SOM_TreeNode node;
		SOM_Declaration currDecl, toAddDecl;
		while (enum.hasMoreElements()) {
			currDecl = (SOM_Declaration)enum.nextElement();

			// Modified by Cindy - March 15
			if (currDecl instanceof SOM_TypeDeclaration) 
				node = new SOM_TreeNode((SOM_TypeDeclaration)currDecl);
			else
				node = new SOM_TreeNode(currDecl.getType());
			toReturn.addElement(node);
		}
		setTypeList(toReturn);
  }

	/** Overrides the semCheck in AbstractExpression
	 * @exception SOM_MethodNotSupportedException if called because it doesnot
	 * make sense to call semCheck(). On SOM_Literal objects setDeclarations()
	 * should be invoked
	 */
	public void semCheck() {
		throw new SOM_MethodNotSupportedException(getClassName() + " does not support semCheck()");
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		// The set should have resulted in a single type.
		SOM_TreeNodeList typeList = getTypeList();
		if (typeList.countElements() != 1) 
			throw  new SYM_UnresolvableException
				("Cannot resolve final type for expression in Literal");
		
		// else the final type is the sole element of the typeList.
		SOM_TreeNode onlyNode = (SOM_TreeNode)typeList.first();
		setResolvedType(onlyNode.getResultType());
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		// typeIn should be one of the types in typeList

		ListEnumeration typeEnum = getTypeList().elements();
		SOM_TreeNode currNode;
		SOM_TypeDeclaration currType;

		while (typeEnum.hasMoreElements()) {
			currNode = (SOM_TreeNode)typeEnum.nextElement();
			currType = currNode.getResultType();

			if (SOM_TypeDeclaration.resolveFinalType(currType, typeIn)) {
				setResolvedType(currType);
				return;
			}
		}

		// typeIn was not found in the overload resovled types.
		throw  new SYM_UnresolvableException
			("Cannot resolve final type for expression in Literal");
	}

	/** Overrides the getNames from SOM_AbstractExpression - same implementation
	 * as in SOM_Name
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
		ListEnumeration declEnum = declList.elements();
		SOM_TypeDeclaration resolvedType = getResolvedType();
		SOM_Declaration currDecl;
		while (declEnum.hasMoreElements()) {
		  currDecl = (SOM_Declaration)declEnum.nextElement();
			if(!SOM_TypeDeclaration.isDistinguishable
				 (currDecl.getType(),resolvedType))
				return new SOM_DeclarationList(currDecl);
			// Note that we can afford to return since any ambiguous multiple
			// type would have been eliminated by the overload resolution algo
			// hence there can be only one decl corresponding to the resolved type
		}
		// shouldn't reach here since the resolved type HAS to be one from the
		// declList. But if we do there is something horribly wrong. So throw
		// an Error
		throw  new RuntimeException("Something horribly wrong with locating proper declaration for resolved type");
	}

	/** To get the value associated with the literal. This should be overridden
	 * in all the non-abstract child classes
	 * @return The Object for the value associated with the literal
	 */
	public abstract Object getValue();

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public abstract Expression M__Value(Factory romFactory,BaseObject
										parent,Universe root);

	/** The publishSOM routine for this class
	 * @return void
	 */
	public abstract void publishSOM(PrintWriter pw);

	/** dummy functions added by krishna and makarand on Mar 7, 2000
     */
	public SOM_Label getLabel() {
		return (SOM_Label)null;
	}
		
	public SOM_Expression getLabelExpression() {
		return (SOM_Expression)null;
	}

		/** The publishPVS routine for this class
		 * @return void
		 */
		public abstract void publishPVS(PrintWriter pw);	
}

