//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_Operator.java,v 1.5 2000/02/10 00:37:28 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_Operator.java,v $
//  Revision 1.5  2000/02/10 00:37:28  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  1999/11/15 22:59:16  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.3  1999/11/13 20:01:28  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.2  1999/11/05 23:43:11  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:46  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:51:16  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.3  1999/07/14 23:24:08  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.2  1999/07/13 19:29:10  rkamath
//  Moved the List and ListEnumeration code to a new util directory
//  Mode the operators as children of AbstractExpression
//  Introduced the FunctionDeclaration and made OperatorDeclaration its child
//
//  Revision 1.1  1999/06/25 18:07:21  rkamath
//  Added the Operators and the Terminal interface (changed from a abstract class
//  to an interface)
//
//
//=============================================================================
package SOM;

/** This represents the <B>Operator</B> object.
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public abstract class SOM_Operator
extends SOM_AbstractExpression {

	/** The declarations corresponding to this operator */
	private SOM_DeclarationList decls;

	/** This will store the SOM_TreeNodeList for each argument */
	protected SOM_ArgumentList domainList = new SOM_ArgumentList();

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_Operator() {
		super();
	}

	/** To set the declarations.
	 * @param declsIn The SOM_DeclarationList to be set
	 * @return void
	 */
	public void setDeclarations(SOM_DeclarationList declsIn) {
		decls = declsIn;
	}	

	/** To get the declarations
	 * @return the SOM_DeclarationList with the decls for this object
	 */
	public SOM_DeclarationList getDeclarations() {
		return decls;
	}

	/** To add a SOM_TreeNodeList to the domainList - Don't really need 
	 * this function since domainList is protected...
	 * @param treeNodeListIn The SOM_TreeNodeList to be added
	 * @return void
	 */
	protected void addToDomainList(SOM_TreeNodeList treeNodeListIn) {
		domainList.addElement(treeNodeListIn);
	}

	/** Overrides the getNames from SOM_AbstractExpression. This throws an
	 * exception if invoked
	 * @return Not Applicable
	 * @exception SOM.SOM_MethodNotSupportedException if invoked
	 */
	public SOM_DeclarationList getNames() {
		throw new SOM_MethodNotSupportedException(getClassName() + " does not support getNames()");
	}	
}

