//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_PackageDeclarativeBlock.java,v 1.3 2000/12/05 03:05:03 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_PackageDeclarativeBlock.java,v $
//  Revision 1.3  2000/12/05 03:05:03  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.2  2000/11/06 18:12:01  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.1  2000/02/13 01:15:55  cvs
//  Completed the implementation for package declarations, and the
//  "use" clause. 
//  Modified the grammar to incorporate these changes. Also populated
//  the object 
//  model with these new objects and completed the pretty-printing for
//  these new 
//  constructs.
//  Also modified the parser to now recognize the keyword "type" as a type.
//  These above changes have been compiled and tested successfully.
//  - Krishna.
//
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

import java.io.PrintWriter;

/** This represents the <B>PackageDeclarativeBlock</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_PackageDeclarativeBlock
extends SOM_BaseClass {

	private SOM_PackageDeclarativeItemList itemList = 
		new SOM_PackageDeclarativeItemList(); 

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_PackageDeclarativeBlock() {
		super();
	}

	/** To add a PackageDeclarativeItem to the itemList
	 * @param itemIn The SOM_PackageDeclarativeItem to be added to the itemList
	 * @return void
	 */
	public void addToPackageDeclarativeItemList
		(SOM_PackageDeclarativeItem itemIn) {
		itemList.addElement(itemIn);
	}

	/** To retrieve the listEnumeration of the itemList
	 * @return The ListEnumeration corresponding to the itemList
	 */
	public ListEnumeration items() {
		return itemList.elements();
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		ListEnumeration itemEnum = this.items();
		SOM_PackageDeclarativeItem item;

		while (itemEnum.hasMoreElements()) {
			item = (SOM_PackageDeclarativeItem)itemEnum.nextElement();
			item.publishSOM(pw);
		}
	}
}

