//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_PackageDefinition.java,v 1.5 2000/11/06 18:12:01 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_PackageDefinition.java,v $
//  Revision 1.5  2000/11/06 18:12:01  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.4  2000/09/26 04:15:43  cvs
//  Added some new meta functions that could be used in the loader
//  --srinivas
//
//  Revision 1.3  2000/05/18 18:03:46  cvs
//  Due to /projects directory failing and the latest version available is old, I
//  am committing the files again.
//  -- Cindy
//
//  Revision 1.3  2000/05/05 19:28:34  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.2  2000/02/17 01:27:05  cvs
//  Implemented the facet expression grammar to incorporate facet composition.
//  Also updated the "use" clause with the above changes.
//  - krishna.
//
//  Revision 1.1  2000/02/13 01:15:56  cvs
//  Completed the implementation for package declarations, and the
//  "use" clause. 
//  Modified the grammar to incorporate these changes. Also populated
//  the object 
//  model with these new objects and completed the pretty-printing for
//  these new 
//  constructs.
//  Also modified the parser to now recognize the keyword "type" as a type.
//  These above changes have been compiled and tested successfully.
//  - Krishna.
//
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;

/** This represents the <B>PackageDefinition</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 */
public class SOM_PackageDefinition
extends SOM_BaseClass {

	/** Local data members */
	SOM_PackageDeclarativeBlock declBlock = null; //optional
	SOM_ExportBlock exportBlock = null; //optional
	SOM_FacetExpression domainExpression = null; // since it may be
	// the null keyword 

	/* This list will store the declarations of this package's parents as
		 given by the domainexpression */
	SOM_DeclarationList parentDeclList = null;

	/** The primary constructor
	 * @return Not applicable
	 */
	public SOM_PackageDefinition() {
		super();
	}

	/** To set the declBlock
	 * @param declBlockIn The SOM_PackageDeclarativeBlock to be set
	 * @return void
	 */
	public void setPackageDeclarativeBlock
		(SOM_PackageDeclarativeBlock declBlockIn) { 
		declBlock = declBlockIn;
	}

    /**added by srinivas to get the declBlock
     * @param void
     * @return SOM_PackageDeclarativeBlock
     */
    public SOM_PackageDeclarativeBlock getPackageDeclarativeBlock(){
		return declBlock;
    }

	/** To set the exportBlock
	 * @param exportBlockIn The SOM_ExportBlock to be set
	 * @return void
	 */
	public void setExportBlock(SOM_ExportBlock exportBlockIn) {
		exportBlock = exportBlockIn;
	}

	/** To get the exportBlock
	 * @return The SOM_ExportBlock of this package definition
	 */
	public SOM_ExportBlock getExportBlock() {
		return exportBlock;
	}
		
	/** To set the domainExpression. Note assumes that the domain expression
	 * has been typechecked and resolved to type facet
	 * @param expressionIn The SOM_Expression object to be set
	 * @return void
	 */
	public void setDomainExpression(SOM_FacetExpression expressionIn) {
		domainExpression = expressionIn;
		// notice the fresh creation of the parentDeclList!
		// The alternative is to concat it to any already existing
		// parentDeclList 
		// the implementation will depend on the semantics of facet expressions
		parentDeclList = domainExpression.getNames();
	}

    /**added by srinivas on sept25th, 2000 to get the domain
     * expression
     * @param void
     * @return SOM_FacetExpression
     */
    public SOM_FacetExpression getDomainExpression(){
		return domainExpression;
    }

	/** To get the parentDeclList
	 * @return SOM_DeclarationList that is the parentDeclList
	 */
	public SOM_DeclarationList getParentDeclarationList() {
		return parentDeclList;
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		
		pw.print("BEGIN ");
		if (domainExpression == null) {
			pw.println("null ");
		} else {
			domainExpression.publishSOM(pw);
			pw.println();
		}

		if (declBlock != null) {
			declBlock.publishSOM(pw);
		}
		if (exportBlock != null) {
			exportBlock.publishSOM(pw);
		}
		pw.print("END ");
	}

}







