//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//           Cindy ckong@ittc.ukans.edu
//
//  $Id: SOM_PredefUnaryFunctionName.java,v 1.12 2001/03/21 00:05:06 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_PredefUnaryFunctionName.java,v $
//  Revision 1.12  2001/03/21 00:05:06  cvs
//  Added the "design" mode for parameters (SOM_Design.java). Changed the
//  semantics for export clauses. Removed the expression member from
//  SOM_FacetDeclaration.java and SOM_PackageDeclaration.java. Added code
//  to catch error when type-checking fails. Removed the redundant methods
//  getExportList() from SOM_FacetDeclaration.java,
//  SOM_PackageDeclaration.java and SOM_ComponentDeclaration.java.
//  -- Cindy
//
//  Revision 1.11  2001/03/12 23:54:39  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.10  2000/11/18 00:18:28  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.9  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the
//  terms(SOM_LabelledExpression) and it returns a ROM Expression. 
//  --srinivas
//
//  Revision 1.8  2000/08/01 22:48:54  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.7  2000/06/22 17:22:46  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.6  2000/04/06 19:13:23  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.5  2000/04/05 07:25:47  cvs
//  The at and tick operators have been added .There were some changes
//  to the predefined unary function name .The grammar has been
//  changed and some classes ahve been added like SOM_AtOperator
//  SOM_TickOperator and SOM_AtTickExpression. 
//  --srinivas
//
//  Revision 1.4  2000/03/23 00:22:49  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.3  2000/02/10 00:37:29  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  2000/02/08 18:42:41  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.1  2000/01/31 19:45:58  cvs
//  Adding unary operator classes and the class for PredefUnaryFunctionName.
//  -- Cindy
//
//
//
//=============================================================================
package SOM;
import ROM.*;
import java.io.PrintWriter;

/** This represents the <B>PredefUnaryFunctionName</B> object
 * @author <I>Roshan Kamath and Cindy Kong</I> (rkamath@ececs.uc.edu,
 * ckong@ittc.ukans.edu) 
 */
public class SOM_PredefUnaryFunctionName
extends SOM_ExpressionWithMonoadicOperator 
implements SOM_Factor {

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_PredefUnaryFunctionName() {
		super();
	}

	/** To set the operator
	 * @param opIn The SOM_MonoadicOperator to be set
	 * @return void
	 */
	public void setOperator(SOM_MonoadicOperator opIn) {
		setMonoadicOperator(opIn);
	}
  
	/** To set the factor
	 * @param factorIn The SOM_Factor to be set
	 * @return void
	 */
    public void setFactor(SOM_Factor factorIn) {
		setStrongerExpression((SOM_AbstractExpression)factorIn);
	}

    /** To get the factor
	 * @return The SOM_Factor for this expression
	 */
	public SOM_Factor getFactor() {
		return (SOM_Factor)getExpression();
	}

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root){
		Formula retExpr = new Formula(parent);
		String str = getMonoadicOperator().toString();
		Type operType = 
			root.getType(getMonoadicOperator().getResolvedType().toString());
		OperatorReference operRef = new
			OperatorReference(retExpr, root.labelLookup(str), operType);
		retExpr.setOperator(operRef);
		Expression expr = getFactor().M__Value(romFactory, retExpr, root);
		retExpr.setRightOp(expr);
		return retExpr;
	}
	
	
	/** publishSOM routine to print the object module
     * @param PrintWriter
     * @return void - mak 
     */
    public void publishSOM(PrintWriter pw){
		getMonoadicOperator().publishSOM(pw);
		getFactor().publishSOM(pw);
    }

	/** dummy functions added by krishna and makarand on Mar 7, 2000
     */
	public SOM_Label getLabel() {
		return (SOM_Label)null;
	}
		
	public SOM_Expression getLabelExpression() {
		return (SOM_Expression)null;
	}
		
	/** publishPVS routine to print the object module
     * @param PrintWriter
     * @return void - mak 
     */
    public void publishPVS(PrintWriter pw){
		getMonoadicOperator().publishPVS(pw);
		getFactor().publishPVS(pw);
    }
}






