//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//
//  $Id: SOM_RecordItemDeclaration.java,v 1.1 2000/11/06 18:12:01 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_RecordItemDeclaration.java,v $
//  Revision 1.1  2000/11/06 18:12:01  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.2  2000/07/24 23:41:02  cvs
//  Some type checking added for record and sequence.
//  -- Cindy
//
//  Revision 1.1  2000/05/18 18:03:48  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I am committing the files again.
//  -- Cindy
//
//  Revision 1.1  2000/05/05 19:28:36  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

/** This represents the <B>RecordItemDeclaration</B> object
 * @author <I>Cindy Kong</I> (ckong@ittc.ukans.edu)
 */
public class SOM_RecordItemDeclaration
extends SOM_Declaration {

	protected SOM_Identifier recordName;
	// to save the fields of the record
	protected SOM_DeclarationList fieldList = new SOM_DeclarationList();

	/** The primary constructor
	 * @param recordNameIn The SOM_Identifier for the name of the declaration
	 * @param myType The SOM_TypeDeclaration corresponding to the
	 * recordMetaType
	 * @return Not applicable
	 */
	public SOM_RecordItemDeclaration(SOM_Identifier recordNameIn,
								   SOM_TypeDeclaration myType) {
		super();
		setKind(SOM_Declaration.RECORD);
		recordName = recordNameIn;
		setType(myType);
	}

	/** To get the string representation of this object. Overrides the
	 * method in SOM_Declaration
	 * @return String representation of this object
	 */
	public String toString() {
		return recordName.toString();
	}

	/** To add to the fieldList
	 * @param fieldIn The SOM_VariableDeclaration corresponding to (one of)
	 * the fields of this record
	 * @return void
	 */
	public void addToFieldList(SOM_VariableDeclaration fieldIn) {
		fieldList.addElement(fieldIn);
	}

	/** To get the fieldList
	 * @return The SOM_DeclarationList of the domains for this record
	 */
	public SOM_DeclarationList getFieldList() {
		return fieldList;
	}

	/** To implement my own version of isDistinguishable for types
	 * @param declOne The first SOM_RecordDeclaration
	 * @param declTwo The next SOM_RecordDeclaration to be distinguished
	 * against declOne
	 * @return true if they are distinguishable, false otherwise
	 */
	public static boolean isDistinguishable
		(SOM_RecordDeclaration declOne, 
		 SOM_RecordDeclaration declTwo) { 
		SOM_DeclarationList fieldListOne = declOne.getFieldList();
		SOM_DeclarationList fieldListTwo = declTwo.getFieldList();

		/*
 		if (fieldListOne.countElements() != fieldListTwo.countElements())
	 		return true;
		ListEnumeration iterOne = fieldListOne.elements();
	 	for (int i = 0; i < fieldListOne.countElements(); i++) {
		 	if (!fieldListTwo.contains(iterOne.nextElement()))
			 	return true;
	 	}
		return false;
	 	*/
		SOM_RecordType typeOne = (SOM_RecordType)declOne.getType();
		SOM_RecordType typeTwo = (SOM_RecordType)declTwo.getType();
		if (!declOne.toString().equalsIgnoreCase(declTwo.toString()))
			return true;
		else
			return(SOM_RecordType.isDistinguishable(typeOne, typeTwo));
	}

	/** To ascertain "compatiblity" for bindings of the kind lhs <- rhs
	 * => RHS must be a subtype of LHS
	 * @param declLHS The recordDecl corresponding to the lhs object
	 * @param declRHS The recordDecl corresponding to the rhs object
	 * @return true, if the rhs is assignable to the lhs, false otherwise
	 */
	public static boolean
		isLeftCompatibleWithRight(SOM_RecordDeclaration declLHS,
								  SOM_RecordDeclaration declRHS) { 
		SOM_RecordType typeLHS = (SOM_RecordType)declLHS.getType();
		SOM_RecordType typeRHS = (SOM_RecordType)declRHS.getType();

		return SOM_RecordType.isLeftTypeCompatibleWithRightType(typeLHS, typeRHS);
	}

	/*
	public static boolean isLeftCompatibleWithRight
		(SOM_RecordDeclaration declLHS,
		 SOM_RecordDeclaration declRHS) { 
		SOM_DeclarationList fieldListLHS = declLHS.getFieldList();
		SOM_DeclarationList fieldListRHSOrig = declRHS.getFieldList();
		SOM_DeclarationList fieldListRHS = (SOM_DeclarationList)
			fieldListRHSOrig.clone();
		int index = fieldListLHS.countElements();

		if (index != fieldListRHS.countElements())
			// if the count of fields is not same, no compatiblity possible
			return false;

		ListEnumeration iterLHS = fieldListLHS.elements();
		ListEnumeration iterRHS;
		SOM_VariableDeclaration fieldLHSDecl, fieldRHSDecl;
		String fieldLHSId;

		// Check for subtyping of field!! The declRHS should be a subtype of
		// declLHS. This part codes the semantics of function subtyping..
		// The current semantics are
		// S::record[d::Ds|r::Rs] is a subtype of F::record[d::Df|r::Rf] iff
		//   Ds is supertype of Df
		//   Rs is a subtype of Rf
		// I have to check for RHS to be a subtype, so S === RHS, F === LHS
		// for the field
		while (iterLHS.hasMoreElements()) {
			// Take first field of LHS record. Find the corresponding
			// field for RHS. Check that type of field in RHS is a
			// subtype of the one in LHS.
            fieldLHSDecl = (SOM_VariableDeclaration)
				iterLHS.nextElement(); 
			fieldLHSId = fieldLHSDecl.toString();
			
			for (int i=0;i<index;i++) {
				fieldRHSDecl = (SOM_VariableDeclaration)
					fieldListRHS.get(i);
				if (fieldRHSDecl.toString().equals(fieldLHSId))
					if (!fieldRHSDecl.getType().isPredicateSubtypeOf
						(fieldLHSDecl.getType()))
						return false;
					else break;
				else
					// The field does not match and we have reached
					// the end of the record
					if (i == index - 1)
						return false;
			}
		}
		return true;
	}				      
	*/
}
