//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_ReferencedName.java,v 1.10 2000/11/18 00:18:28 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_ReferencedName.java,v $
//  Revision 1.10  2000/11/18 00:18:28  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.9  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the terms(SOM_LabelledExpression) and it returns a ROM Expression.
//  --srinivas
//
//  Revision 1.8  2000/08/01 22:48:55  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.7  2000/06/22 17:22:48  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.6  2000/04/06 19:13:24  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.5  2000/02/10 00:37:30  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  2000/02/08 18:42:41  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.3  1999/11/17 18:31:55  cvs
//  Coded resolution of final type (unfolding overload resolution) and included
//  actions for TCC generation
//
//  Revision 1.2  1999/11/13 20:01:28  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/11/11 00:04:24  cvs
//  Fresh changes as part of the streamlining process with new
//  object model and grammar
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;
import java.io.StringWriter;
import ROM.*;
/** This represents the <B>ReferencedName</B> object
 * <refereced_name> ::== <name> \[ <array_listing> \]
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_ReferencedName
extends SOM_Name {

	/** The name in this formalName */
	private SOM_Name name;
	/** The array listing for this Referenced Name */
	private SOM_ArrayListing arrayListing;
	 

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_ReferencedName() {
		super();
	}

	/** To set the name
	 * @param nameIn The SOM_Name to be set
	 * @return void
	 */
	public void setName(SOM_Name nameIn) {
		name = nameIn;
	}

	/** To get the name - overrides the getName in SOM_Name
	 * @return SOM_Name that is the name in this formalName
	 */
	public SOM_Name getName() {
		return name;
	}

	/** To set the arrayListing
	 * @param arrayListIn The SOM_ArrayListing to be set
	 * @return void
	 */
	public void setArrayListing(SOM_ArrayListing arrayListIn) {
		arrayListing = arrayListIn;
	}

	/** To get the arrayListing
	 * @param arrayListIn The SOM_ArrayListing to be set
	 * @return void
	 */
	public SOM_ArrayListing getArrayListing() {
		return arrayListing;
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		throw new RuntimeException(getClassName() + ": Yet to implement resolveFinalType");
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		throw new RuntimeException
			(getClassName() + ": Yet to implement resolveFinalType");
	}

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root){
		return null;
	}

	/** publishSOM routine 
     * @return void
     */
    public void publishSOM(PrintWriter pw){
		name.publishSOM(pw);
		arrayListing.publishSOM(pw);
    }

		/** The publishPVS routine for this class
		 * @return void
     * To be implemented as needed */
		public void publishPVS(PrintWriter pw) {
        publishSOM(pw);
		}
}
