//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy Kong ckong@ittc.ukans.edu
//
//  $Id: SOM_SetDeclarationStatement.java,v 1.3 2000/12/03 23:55:26 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_SetDeclarationStatement.java,v $
//  Revision 1.3  2000/12/03 23:55:26  cvs
//  Added the M__Value method to the Literal classes.Previously they were returning null.The M__Type method for all the Declaration statement classes has been completed
//  --srinivas
//
//  Revision 1.2  2000/07/24 23:41:04  cvs
//  Some type checking added for record and sequence.
//  -- Cindy
//
//  Revision 1.1  2000/07/11 00:11:15  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are keywords. Typechecking for set is done. Added SetType
//  and FunctionType. 
//  False and true are predefined as boolean.
//  -- Cindy
//
//
//============================================================================
package SOM;

import util.ListEnumeration;
import java.io.*;
import java.util.*;
import ROM.*;

public class SOM_SetDeclarationStatement 
extends SOM_BaseClass {

	private SOM_Expression typeExpr = null;
	private SOM_Expression valueExpr = null;
	
	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_SetDeclarationStatement()
	{
		super();
	}

	/** To set the expression for the type
	 * @param typeExprIn The SOM_Expression representing the type
	 * @return void
	 */
	public void setType(SOM_Expression typeExprIn) {
		typeExpr = typeExprIn;
	}

	/** To get the expression for the type
	 * @return The SOM_Expression representing the type
	 */
	public SOM_Expression getType() {
		return typeExpr;
	}
 
	/** To set the value
	 * @param valueExprIn The SOM_Expression representing the values
	 * @return void
	 */
	public void setValueExpr(SOM_Expression valueExprIn) {
		valueExpr = valueExprIn;
	}

	/** To get the expression for the values
	 * @return The SOM_Expression for the values in the type
	 */
	public SOM_Expression getValueExpr() {
		return valueExpr;
	}

	//Added by srinivas on Nov 29th, 2000
	/** The function is to get the Type of the set declaration
	 * @param parent 
	 * @param root The universe Object to search for predefined types
	 * @param value The Expression Object to add to the bunchValue of type
	 * @return ROM.Type
	 */
	public Type M__Type(BaseObject parent,Universe root, Expression value){
		SOM_Expression expr = getType();
		Type type = null;
		if (expr != null){
			String str = expr.getResolvedType().toString();
			if (str.equals("integer")){
				type = root.getType("setInteger");
				if (value != null)
					((SetIntegerType)type).addOneElement(value);
			}else if (str.equals("real")){
				type = root.getType("setReal");
				if (value != null)
					((SetRealType)type).addOneElement(value);
			}else if (str.equals("bit")){
				type = root.getType("setBit");
				if (value != null)
					((SetBitType)type).addOneElement(value);
			}else if (str.equals("boolean")){
				type = root.getType("setBoolean");
				if (value != null)
					((SetBooleanType)type).addOneElement(value);
			}else{
				Type result = null;
				if (parent instanceof Facet){
					type = ((Facet)parent).getType(str);
					result =
						((Facet)parent).alreadyDeclared("set",type);
				}
				if (type == null){
					type = root.getType(str);
					result = root.alreadyDeclared("set",type);
				}
				if (result == null){
					BunchValue setBunch =
						new BunchValue(null, null, new HashSet());
					SetType set = new
						SetType(parent,setBunch,type);
					setBunch.setType(set);
					if (value != null)
						((SetType)set).addOneElement(value);
					if (parent instanceof Facet)
						((Facet)parent).addTypeElement(set);
					return (Type)set;
				}else {
					if (value != null)
						((SetType)result).addOneElement(value);
					return result;
				}
			}
		}
		return type;
	}


	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw){
		pw.print(" SET");
		pw.print("(");
		typeExpr.publishSOM(pw);
		pw.print(")");
	    if (valueExpr != null){
			pw.print(" IS");
			valueExpr.publishSOM(pw);
	    }
	}
}
