//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_Terminal.java,v 1.3 2000/02/10 00:37:33 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_Terminal.java,v $
//  Revision 1.3  2000/02/10 00:37:33  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  1999/11/05 23:43:12  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:50  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:51:44  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.2  1999/07/13 14:27:47  rkamath
//  Changed the publishRosetta() function to take in a PrintWriter
//  Added some new classes too.
//  Also commented out currency based operations from the SOM_List class. Hope-
//  fully at a later date I'll include them in the SOM_ListEnumeration class if
//  found necessary.
//
//  Revision 1.1  1999/06/25 18:07:17  rkamath
//  Added the Operators and the Terminal interface (changed from a abstract class
//  to an interface)
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;

/** This represents the <B>Terminal</B> object. 
 * Note: All the objects that implement Terminal must ensure that they
 * override the publishRosetta(PrintWriter) routine in SOM_BaseClass.
 * Note however that since SOM_BaseClass itself provides a body for the
 * function, the compiler CANNOT detect if every terminal object has
 * overriden this abstract function...
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu) 
 * @see SOM.SOM_BaseClass */
public interface SOM_Terminal {
	/** The function that needs to be overridden by each class that
	 * implements the Terminal interface.
	 * @return void
	 */
	public abstract void publishRosettaTerminal(PrintWriter printOut);
}
