//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_TreeNodeList.java,v 1.4 2000/02/10 00:37:34 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_TreeNodeList.java,v $
//  Revision 1.4  2000/02/10 00:37:34  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.3  1999/11/15 03:16:29  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.2  1999/11/13 20:01:29  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/10/27 20:43:50  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:51:47  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.2  1999/10/01 15:23:35  rkamath
//  Commit for record - more cleaner typing ideas.
//
//  Revision 1.1  1999/08/01 17:00:12  rkamath
//  Added new files for overload resolution
//
//=============================================================================
package SOM;

import util.List;
import util.ListEnumeration;

/** This represents the <B>TreeNodeList</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_TreeNodeList
extends List {

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_TreeNodeList() {
		super();
	}

	/** The Second Constructor
	 * @param nodeIn The first SOM_TreeNode object to be added
	 * @return Not applicable
	 */
	public SOM_TreeNodeList(SOM_TreeNode nodeIn) {
		super(nodeIn);
	}

	/** The Third constructor
	 * @param declListIn The declList for which a SOM_TreeNodeList is to be made
	 * @return Not applicable
	 */
	public SOM_TreeNodeList(SOM_DeclarationList declList) {
		super();
		ListEnumeration declEnum = declList.elements();
		while (declEnum.hasMoreElements())
			addElement(new SOM_TreeNode
								 ((SOM_TypeDeclaration)declEnum.nextElement()));
		// note that this can possibly throw a illegal type cast exception - 
		// especially if the declList contains declarations other than
		// SOM_TypeDeclaraiton
		
	}

	/** To add a SOM_TreeNode to the list
	 * @param nodeIn The SOM_TreeNode object to be added
	 * @return void
	 */
	public void addElement(SOM_TreeNode nodeIn) {
		super.addElement(nodeIn);
	}

	/** To remove the ambiguous treeNodes
	 * @return void
	 */
	public void removeAmbiguousTreeNodes() {
		// first make a bag of all ambiguous nodes and then sequentially
		// remove them
		SOM_TreeNode node;
		SOM_TreeNodeList ambigNodes = new SOM_TreeNodeList();

		// FIrst collect all the ambiguous nodes in ambigNodes
		ListEnumeration enum = elements();
		while (enum.hasMoreElements()) {
			node = (SOM_TreeNode)enum.nextElement();
			if (node.getIsAmbiguous()) ambigNodes.addElement(node);
		}

		// Now remove each ambig node
		enum = ambigNodes.elements();
		while (enum.hasMoreElements()) {
			removeElement((SOM_TreeNode)enum.nextElement());
		}		
	}
}

