//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_TypeDeclaration.java,v 1.18 2001/04/04 22:23:20 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_TypeDeclaration.java,v $
//  Revision 1.18  2001/04/04 22:23:20  cvs
//  Removed an error in typechecking of record. Type checking now works
//  for recursive functions.
//  -- Cindy
//
//  Revision 1.17  2000/11/06 18:12:02  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.16  2000/08/18 19:50:48  cvs
//  Added some semantic checking to use of underscore as parameter to
//  function.
//  -- Cindy
//
//  Revision 1.15  2000/08/02 23:45:32  cvs
//  Taken care of a null exception being thrown.
//  -- Cindy
//
//  Revision 1.14  2000/07/24 23:41:04  cvs
//  Some type checking added for record and sequence.
//  -- Cindy
//
//  Revision 1.13  2000/07/11 00:11:16  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are keywords. Typechecking for set is done. Added SetType
//  and FunctionType. 
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.12  2000/06/22 17:22:50  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.11  2000/05/18 18:03:50  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I am committing the files again.
//  -- Cindy
//
//  Revision 1.10  2000/04/28 19:34:08  cvs
//  Added some more of typechecking. Added parsing of the $ operator
//  and constant literal. 
//  -- Cindy
//
//  Revision 1.9  2000/04/11 03:17:52  cvs
//  The dummy M__Label and M__Value functions have been removed.
//  --srinivas
//
//  Revision 1.8  2000/04/06 19:13:25  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.7  2000/03/23 00:22:50  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.6  2000/02/13 01:15:56  cvs
//  Completed the implementation for package declarations, and the
//  "use" clause. Modified the grammar to incorporate these
//  changes. Also populated the object model with these new objects
//  and completed the pretty-printing for these new constructs.
//  Also modified the parser to now recognize the keyword "type" as a type.
//  These above changes have been compiled and tested successfully.
//  - Krishna.
//
//  Revision 1.5  2000/02/10 00:37:34  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  2000/02/08 18:42:45  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.3  1999/11/17 18:31:55  cvs
//  Coded resolution of final type (unfolding overload resolution) and included
//  actions for TCC generation
//
//  Revision 1.2  1999/11/13 20:01:29  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/10/27 20:43:51  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:51:50  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.7  1999/10/01 15:23:36  rkamath
//  Commit for record - more cleaner typing ideas.
//
//  Revision 1.6  1999/08/01 20:54:39  rkamath
//  Made a lot of changes with regard to types and checking. I'm convinced that
//  there are going to me hundreds more changes until I stabilize on how to
//  import and [de]serialize
//
//  Revision 1.5  1999/07/28 01:59:46  rkamath
//  Changed all the SOM_Declaration references to SYM_SymboltableEntry as far
//  as lookUp and typeResolution are concerned
//
//  Revision 1.4  1999/07/16 21:50:35  rkamath
//  Changed everyexpression to point to the type declaration instead of having
//  a private variable which would indicate the resulting type
//
//  Revision 1.3  1999/07/13 14:27:49  rkamath
//  Changed the publishRosetta() function to take in a PrintWriter
//  Added some new classes too.
//  Also commented out currency based operations from the SOM_List class. Hope-
//  fully at a later date I'll include them in the SOM_ListEnumeration class if
//  found necessary.
//
//  Revision 1.2  1999/07/10 21:31:26  rkamath
//  The commit for today. Introduced a toString() function in
//  SOM_Declaration that will be used to hash "declarations" into the
//  symboltable 
//
//  Revision 1.1  1999/07/09 22:21:22  rkamath
//  A lot of changes to incorporate the intent of type checking
//
//
//=============================================================================
package SOM;

import java.util.NoSuchElementException;

import java.io.PrintWriter;

/** This represents the <B>TypeDeclaration</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_TypeDeclaration
extends SOM_Declaration {

	private SOM_Identifier id;
	private SOM_TypeExpression typeExpr = null; //since it is optional

	// for subtyping purposes....
	private boolean isSubType = false;
	private SOM_Name parentTypeName = null; // if null,
	// uninterpreted/predefined 
	//private transient SOM_TypeDeclaration parentType = null;
	// Removed "transience"  Cindy May 24, 2000
	private SOM_TypeDeclaration parentType = null;

	// the root of the type-hierarchy tree...
	private SOM_TypeDeclaration baseType = null;
	private transient SOM_Identifier baseTypeName = null;

	/** The primary constructor
	 * @param idIn The SOM_Identifier that is the name of this type
	 * @param myTypeIn This should be the declaration corresponding to
	 * the "bunch metatype" - re semantics document.
	 * @return Not applicable
	 */
	public SOM_TypeDeclaration(SOM_Identifier idIn, 
							   SOM_TypeDeclaration myTypeIn) {
		super();
		setKind(TYPE);
		id = idIn;
		setType(myTypeIn);
	}

	/** To set the identifier for the name of this type
	 * @param idIn The SOM_Identifier to be set
	 * @return void
	 */
	public void setName(SOM_Identifier idIn) {
		id = idIn;
	}

	/** To get the name of this type
	 * @return The SOM_Identifier corresponding to this type name
	 */
	public SOM_Identifier getName() {
		return id;
	}

	/** To set the typeExpression. If the typeExpression is a
	 * subTypeExpression then the kind is set to SUBTYPE and the
	 * parentTypeEntry is also appropriately initialized.
	 * @param typeExprIn The SOM_TypeExpression that is to be set
	 * @return void
	 */
	public void setTypeExpression(SOM_TypeExpression typeExprIn) {
		typeExpr = typeExprIn;
		if (typeExprIn instanceof SOM_SubTypeExpression) {
		  setParentType(((SOM_SubTypeExpression)typeExpr).getType());
		}
	}

	/** To set the parentType
	 * @param parentTypeIn The SOM_TypeDeclaration to be set as the parentType
	 * @return void
	 */
	public void setParentType(SOM_TypeDeclaration parentTypeIn) {
		isSubType = true;
		parentType = parentTypeIn;
		parentTypeName = parentTypeIn.getName();
		baseType = parentTypeIn.getBaseType();
		baseTypeName = baseType.getName();
	}

	/** To get the parentType
	 * @return The SOM_Declaration corresponding to the parentType
	 */
	public SOM_TypeDeclaration getParentType() {
		if (parentTypeName == null)
			// no parentType has been set yet!
			throw new NoSuchElementException(getClassName() + 
						   ": call to getParentType() before setParentType()");
		if (parentType == null) {
			// deserialize it
			System.out.println("Need to deserialize and set the parentType");
		}
		return parentType;
	}

	/** To get the parentType name
	 * @return The SOM_Name corresponding to the name of the parent type
	 */
	public SOM_Name getParentTypeName() {
		return parentTypeName;
	}

	/** To get the fundamental base type
	 * @return The SOM_TypeDeclaration corresponding to the fundamental 
	 * base type
	 */
	private SOM_TypeDeclaration getBaseType() {
		if (baseType == null) return this;
		return baseType;
	}

	/** To know if this type is a subtype of another type
	 * @return true if this type is a subtype of some type, false
	 * otherwise
	 */
	public boolean isASubtype() {
		return isSubType;
	}

	/** To ascertain if this type is a subtype of another type
	 * @param typeIn The SOM_TypeDeclaration which is possibly the supertype of
	 * this type
	 * @return true if this type is a subtype (not just immediate) of
	 * typeIn, false otherwise. Note that it returns true even if typeIn
	 * is the same type as itself!
	 */
	public boolean isSubtypeOf(SOM_TypeDeclaration typeIn) {
		//if (this == typeIn) return true; // trivial case
		//System.out.println("In isSubtypeOf, comparing: " + this.toString()
		//				   + " to: " + typeIn.toString());
        if (!isDistinguishable(this, typeIn)) return true;
		// The underscore type is generic and can be replaced by any
		// type, therefore, it is considered as a subtype of all types
		if (toString().equalsIgnoreCase("underscoreType"))
			return true;
		if (parentTypeName == null)
			// this is a fundamental type
			return false;
		return getParentType().isSubtypeOf(typeIn);	
	}

	/** To get the string representation of this object. Overrides the
	 * method in SOM_Declaration
	 * @return String representation of this object
	 */
	public String toString() {
		return id.toString();
	}


	/** To ascertain if this object is a predicateSubtype of some
	 * other type 
	 * @param declIn The SOM_TypeDeclaration which may be the
	 * superType 
	 * @return true if this object is a predicate subtype of
	 * declIn. false otherwise
	 */
	public boolean isPredicateSubtypeOf(SOM_TypeDeclaration declIn) {
		if (!isDistinguishable(declIn,this)) return true;

		//System.out.println("R.H.S. basetype: " + getBaseType().toString());
		//System.out.println("L.H.S. basetype: " + 
		//				   declIn.getBaseType().toString());
		// They should belong to the same typeHierarchy
		// Cannot compare object to object, but names associated to object
		// When restoring an object from a serialized symbol table,
		// the objects are different even if they mean the same
		// Cindy - May 24, 2000
		//if (getBaseType() != declIn.getBaseType()) return false;
		if (!getBaseType().toString().equals(declIn.getBaseType().toString())){
			//System.out.println
			//	("In isPredicateSubtypeOf, base types are different");
			return false;
		}

		// In the same typeHieararchy return false if declIn is a
		// direct subtype of this (provided that they are two
		// different types! :) 
		if (declIn.isSubtypeOf(this)) {
			//System.out.println("In isPredicateSubtypeOf, left hand" +
			//				   " side is a subtype of right hand !");
			return false;
		}

		// added by Cindy, April 3
		if (isSubtypeOf(declIn)) return true;

		// in all other cases return false!
		return false;

		// in all other cases return true! A TCC will hopefully be
		// generated finally at the topmost level
		//return true;

		// If this is a direct subtype of declIn return true!
		//if (isSubtypeOf(declIn)) return true;

		// Else Need to generate a TCC!!!
		//System.out.println("A TCC is generated for " + toString() +
		// " to be a predicate subtype of " + declIn.toString()); 
		//return true;
		
	}

	/** To implement my own version of isDistinguishable for types
	 * @param declOne The first SOM_TypeDeclaration
	 * @param declTwo The next SOM_TypeDeclaration to be distinguished
	 * against declOne
	 * @return true if they are distinguishable, false otherwise
	 */
	public static boolean isDistinguishable(SOM_TypeDeclaration
											declOne,
											SOM_TypeDeclaration
											declTwo) { 
		if (declOne == null || declTwo == null)
			return true;

		// Currently atleast, two types are indistinguishable if their
		// declarations differ....
		//return (declOne != declTwo);
		if (declOne instanceof SOM_SetType && declTwo instanceof SOM_SetType)
			return (SOM_SetType.isDistinguishable
					((SOM_SetType)declOne, (SOM_SetType)declTwo));

		if (declOne instanceof SOM_ArrayType
			&& declTwo instanceof SOM_ArrayType)
			return (SOM_ArrayType.isDistinguishable
					((SOM_ArrayType)declOne, (SOM_ArrayType)declTwo));

		if (declOne instanceof SOM_RecordType
			&& declTwo instanceof SOM_RecordType)
			return (SOM_RecordType.isDistinguishable
					((SOM_RecordType)declOne, (SOM_RecordType)declTwo));

		return (!declOne.toString().equals(declTwo.toString()));
	}

	/** To implement getMoreGeneral for types
	 * @param declOne The first SOM_TypeDeclaration
	 * @param declTwo The next SOM_TypeDeclaration that is to be
	 * compared to declOne
	 * @return resultType, the type between declOne and declTwo that
	 * is the more general  
	 */
	public static SOM_TypeDeclaration getMoreGeneral
		(SOM_TypeDeclaration declOne, 
		 SOM_TypeDeclaration declTwo) {  
		if (!isDistinguishable(declOne, declTwo))
			return declOne;
		if (declOne.isSubtypeOf(declTwo))
			return declTwo;
		if (declTwo.isSubtypeOf(declOne))
			return declOne;
		return null;
	}

	/** To implement getCommonType for types
	 * @param declOne The first SOM_TypeDeclaration
	 * @param declTwo The next SOM_TypeDeclaration that is to be
	 * compared to declOne
	 * @return resultType, the type that is common to declOne and
	 * declTwo in the hierarchy of types.  
	 */
	public static SOM_TypeDeclaration getCommonType
		(SOM_TypeDeclaration declOne, 
		 SOM_TypeDeclaration declTwo) {
		SOM_TypeDeclaration declOneTemp = declOne;
		SOM_TypeDeclaration declTwoTemp = declTwo;
		SOM_TypeDeclaration resultType;

		// declOne is a subtype of declTwo, or vice versa
		resultType = getMoreGeneral(declOne, declTwo);
		if (resultType != null)
			return resultType;

		// Both declOne and declTwo have a common parent type
		while (declOneTemp.isASubtype()) {
			declOneTemp = declOneTemp.getParentType();
			resultType = getMoreGeneral(declTwo, declOneTemp);
			if (resultType != null)
				return resultType;
		}

		while (declTwoTemp.isASubtype()) {
			declTwoTemp = declTwoTemp.getParentType();
			resultType = getMoreGeneral(declOne, declTwoTemp);
			if (resultType != null)
				return resultType;
		}

		return null;
	}

	/** This is the function called by resolveFinalType of
	 * SOM_AbstractExpression and hence is named so
	 * @param manifestType The type to which an expression has evaluated
	 * @param expectedType The type to which we want the expression to
	 * evaluate to 
	 * @return true if the manifest type is compatible with the expected type
	 * false other wise. Note that a TCC may be generated when returning true
	 */
	public static boolean resolveFinalType(SOM_TypeDeclaration
										   manifestType,
										   SOM_TypeDeclaration
										   expectedType) {  
		// return true if manifestType is a predicate subtype of
		// expectedType generate a tcc if needed
		// The  rules are:
		// 1. both should belong to the same typehierarchy, else
		// return false 
		// 2. if expectedType is a subtype of manifestType return
		// false (sem error) 
		// 3. if manifestType is a subtype of expected type then
		// return true else, 
		// generate TCC and return true
		// 4. return false
		
		if (manifestType instanceof SOM_SetType && 
			expectedType instanceof SOM_SetType) {
			manifestType = ((SOM_SetType)manifestType).getMemberType();
			expectedType = ((SOM_SetType)expectedType).getMemberType();
		}

		if (manifestType instanceof SOM_ArrayType && 
			expectedType instanceof SOM_ArrayType) {
			manifestType = ((SOM_ArrayType)manifestType).getMemberType();
			expectedType = ((SOM_ArrayType)expectedType).getMemberType();
		}

		try {
			if (manifestType instanceof SOM_RecordType &&
				expectedType instanceof SOM_RecordType) {
				return SOM_RecordType.isLeftTypeCompatibleWithRightType
					(expectedType, manifestType);
			}
		} catch (NullPointerException e) {
			System.out.println("The two records do not match: " + e);
		}
			
		if (manifestType == null || expectedType == null)
			return false;

		if (!manifestType.isPredicateSubtypeOf(expectedType)) return false;

		// manifestType is a predicate sub type of expectedType
		if (!manifestType.isSubtypeOf(expectedType)) {
			// manifestType is not a direct subtype of expectedType
			// generate TCC here
			System.out.println("TCC Generated: " + manifestType +
							   "should be a predicate subtype of " +
							   expectedType);  
		}
		return true;		
	}
		
	/** publishSOM routine 
     * @return void
     */
	public void publishSOM(PrintWriter pw) {
		id.publishSOM(pw);
	}

}
