//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy Kong ckong@ittc.ukans.edu
//
//  $Id: SOM_TypeDeclarationStatement.java,v 1.8 2001/07/13 22:40:49 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_TypeDeclarationStatement.java,v $
//  Revision 1.8  2001/07/13 22:40:49  cvs
//
//  Added initial implementation of constructed types.
//  Contains function declaration generation only, still needs more
//  semantic checking (type check and resolve type on observer function
//  notably on the subtype selection function)
//  --morel
//
//  Revision 1.7  2001/07/10 22:46:50  cvs
//
//  Added shorthand notation for type declaration
//  type is shorthand for subtype(univ)
//
//  Revision 1.6  2001/03/12 23:54:40  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.5  2001/03/06 05:02:27  cvs
//  The M__Value method has been implemented for SelectedName and some
//  errors in the Meta functions have been removed in the other files 
//  --srinivas
//
//  Revision 1.4  2001/02/01 22:34:30  cvs
//  Added some accessor methods to the classes which are used in DVTG
//  --srinivas
//
//  Revision 1.3  2000/12/05 03:05:04  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.2  2000/12/03 23:55:26  cvs
//  Added the M__Value method to the Literal classes.Previously they
//  were returning null.The M__Type method for all the Declaration
//  statement classes has been completed 
//  --srinivas
//
//  Revision 1.1  2000/07/11 00:11:17  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are 
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//
//============================================================================
package SOM;

import util.ListEnumeration;
import java.io.PrintWriter;
import ROM.*;
import java.util.*;
public class SOM_TypeDeclarationStatement 
extends SOM_BaseClass {

	private String keyWord;

	private SOM_Expression typeExpr = null;
	private SOM_Expression setTypeExpr = null;
	private SOM_Expression sequenceTypeExpr = null;
	private SOM_Expression arrayTypeExpr = null;
	private SOM_Expression typeValue = null;

    // Added by Morel, July 10, 2001
    // Determines a shortcut type
    private boolean isType = false;
    
    // Added by Morel, July 11, 2001
    private SOM_ConstructedTypeDeclarationStatement constType;
	
	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_TypeDeclarationStatement()
	{
		super();
	}

	/** To set the string representing the keyword being used
	 * @param keyWordIn The String representing the keyword
	 * @return void
	 */
	public void setKeyWord(String keyWordIn) {
		keyWord = keyWordIn;
	}

	/** To get the string representing the keyword
	 * @return String, representing keyword being used
	 */
        public String getKeyWord() {
    	        return keyWord;
        }

	/** To set the expression for the type
	 * @param typeExprIn The SOM_Expression representing the type
	 * @return void
	 */
	public void setType(SOM_Expression typeExprIn) {
		typeExpr = typeExprIn;
	}

	/** To get the expression for the type
	 * @return The SOM_Expression representing the type
	 */
    	public SOM_Expression getType() {
    	return typeExpr;
	}
 
	/** To set the expression for set type
	 * @param setTypeExprIn The SOM_Expression representing the set type
	 * @return void
	 */
	public void setSetType(SOM_Expression setTypeExprIn) {
		setTypeExpr = setTypeExprIn;
	}

	/** To get the expression for the set type
	 * @return The SOM_Expression representing the set type
	 */
	public SOM_Expression getSetType() {
		return setTypeExpr;
	}
 
	/** To set the expression for the sequence type
	 * @param sequenceTypeExprIn The SOM_Expression representing the
	 * sequence type 
	 * @return void
	 */
	public void setSequenceType(SOM_Expression sequenceTypeExprIn) {
		sequenceTypeExpr = sequenceTypeExprIn;
	}

	/** To get the expression for the sequence type
	 * @return The SOM_Expression representing the sequence type
	 */
	public SOM_Expression getSequenceType() {
		return sequenceTypeExpr;
	}
 
	/** To set the expression for the array type
	 * @param arrayTypeExprIn The SOM_Expression representing the
	 * array type 
	 * @return void
	 */
	public void setArrayType(SOM_Expression arrayTypeExprIn) {
		arrayTypeExpr = arrayTypeExprIn;
	}

	/** To get the expression for the array type
	 * @return The SOM_Expression representing the array type
	 */
	public SOM_Expression getArrayType() {
		return arrayTypeExpr;
	}
 
	/** To set the expression for the values of the type
	 * @param typeValueIn The SOM_Expression for the values in type
	 * @return void
	 */
	public void setValue(SOM_Expression typeValueIn) {
		typeValue = typeValueIn;
	}

	/** To get the expression for the values of the type
	 * @return The SOM_Expression for the values in the type
	 */
	public SOM_Expression getValue() {
		return typeValue;
	}

    /** To set the shortcut type notation
     * @return void
     */
    public void setIsType() {
	isType = true;
    }

    // Added by Morel, July 11, 2001
    /** To set the constructed type for the values of the type
     * @param constTypeValueIn The SOM_ConstructedTypeDeclarationStatement for the values in type
     * @return void
     */
    public void setConstType(SOM_ConstructedTypeDeclarationStatement constTypeValueIn) {
	constType = constTypeValueIn;
    }

    // Added by Morel, July 11, 2001
    /** To get the constructedType for the values of the type
     * @return The SOM_ConstructedTypeDeclarationStatement for the values in the type
     */
    public SOM_ConstructedTypeDeclarationStatement getConstType() {
	return constType;
    }

	//Added by srinivas on Nov 29th, 2000
	/** The function is to get the Type of the array declaration
	 * @param parent To search for previous user defined types
	 * @param newParent The parent that is the declaration item itself
	 * @param root The Universe Object to search for predefined types
	 * @param identifier The new type name
	 * @param value The bunch value for the new type
	 * @return ROM.Type
	 */
	public Type M__Type(BaseObject parent, BaseObject
						newParent, Universe root, String identifier,
						Expression value){
		Type type = null;
		if (getType() != null){
			String str = getType().getResolvedType().toString();
			type = root.getType(str);
			if (type == null)
				type = ((Facet)parent).getType(str);
		}else if (getSetType() != null){
			if (getSetType() != null){
				String str = getSetType().getResolvedType().toString();
				if (str.equals("integer"))
					type = root.getType("setInteger");
				else if (str.equals("real"))
					type = root.getType("setReal");
				else if (str.equals("bit"))
					type = root.getType("setBit");
				else if (str.equals("boolean"))
					type = root.getType("setBoolean");
				else{
					Type result = null;
					if (parent instanceof Facet){
						type = ((Facet)parent).getType(str);
						result =
							((Facet)parent).alreadyDeclared("set",type);
					}
					if (type == null){
						type = root.getType(str);
						result = root.alreadyDeclared("set",type);
					}
					if (result == null){
						BunchValue setBunch =
							new BunchValue(null, null, new HashSet());
						SetType set = new
							SetType(parent,setBunch,type);
						setBunch.setType(set);
						if (parent instanceof Facet)
							((Facet)parent).addTypeElement(set);
						return (Type)set;
					}else {
						return result;
					}
				}
			}
		}else if (getSequenceType() != null){
			if (getSequenceType() != null){
				String str = getSequenceType().getResolvedType().toString();
				if (str.equals("integer"))
					type = root.getType("sequenceInteger");
				else if (str.equals("real"))
					type = root.getType("sequenceReal");
				else if (str.equals("bit"))
					type = root.getType("sequenceBit");
				else if (str.equals("boolean"))
					type = root.getType("sequenceBoolean");
				else{
					Type result=null;
					if (parent instanceof Facet){
						type = ((Facet)parent).getType(str);
						result =
							((Facet)parent).alreadyDeclared("sequence",type);
					}
					if (type == null){
						type = root.getType(str);
						result = root.alreadyDeclared("sequence",type);
					}
					if (result == null){
						BunchValue sequenceBunch =
							new BunchValue(null, null, new HashSet());
						SequenceType sequence = new
							SequenceType(parent,sequenceBunch,type);
						sequenceBunch.setType(sequence);
						if (parent instanceof Facet)
							((Facet)parent).addTypeElement(sequence);
						return (Type)sequence;
					}else {
						return result;
					}
				}
			}
		}else if (getArrayType() != null){
			if (getArrayType() != null){
				String str = getArrayType().getResolvedType().toString();
				if (str.equals("integer"))
					type = root.getType("arrayInteger");
				else if (str.equals("real"))
					type = root.getType("arrayReal");
				else if (str.equals("bit"))
					type = root.getType("arrayBit");
				else if (str.equals("boolean"))
					type = root.getType("arrayBoolean");
				else{
					Type result = null;
					if (parent instanceof Facet){
						type = ((Facet)parent).getType(str);
						result =
							((Facet)parent).alreadyDeclared("array",type);
					}
					if (type == null){
						type = root.getType(str);
						result = root.alreadyDeclared("array",type);
					}
					if (result == null){
						BunchValue arrayBunch =
							new BunchValue(null, null, new HashSet());
						ArrayType array = new
							ArrayType(parent,arrayBunch,type);
						arrayBunch.setType(array);
						if (parent instanceof Facet)
							((Facet)parent).addTypeElement(array);
						return (Type)array;
					}else {
						return result;
					}
				}
			}
		}

		// Added by Morel, July 10, 2001
		// Handles shortcut type
		else if (isType) {
		    type = root.getType("univ");
		    if (type == null)
			type = ((Facet)parent).getType("univ");
		}

		BunchValue bunchVal;
		if (value != null)
			bunchVal = new BunchValue(newParent,type,value);
		

		else
			bunchVal = type.getElements();
		UserDefinedType newType = new
			UserDefinedType(newParent,identifier,bunchVal,type);
		return newType;
	}
	

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw){
		pw.print(keyWord);
		if (!isType) {
		    pw.print("(");
		    if (typeExpr != null)
			typeExpr.publishSOM(pw);
		    if (setTypeExpr != null) {
			pw.print(" SET");
			pw.print("(");
			setTypeExpr.publishSOM(pw);
			pw.print(")");
		    }
		    if (sequenceTypeExpr != null) {
			pw.print(" SEQUENCE");
			pw.print("(");
			sequenceTypeExpr.publishSOM(pw);
			pw.print(")");
		    }
		    if (arrayTypeExpr != null) {
			pw.print(" ARRAY");
			pw.print("(");
			arrayTypeExpr.publishSOM(pw);
			pw.print(")");
		    }
		    pw.print(")");
		}
		if (typeValue != null){
		    pw.print(" IS ");
		    typeValue.publishSOM(pw);
		}
		
		// Added by Morel, July 11, 2001
		else if (constType != null){
		    pw.print(" IS ");
		    constType.publishSOM(pw);
		}		
	}
}
