//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SYM_StaticFunctions.java,v 1.9 2000/12/05 03:06:28 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SYM_StaticFunctions.java,v $
//  Revision 1.9  2000/12/05 03:06:28  cvs
//  Modified semantic checking, needed after grammar was changed to have
//  all files starting in a package.
//  -- Cindy
//
//  Revision 1.8  2000/07/11 00:11:54  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are keywords. Typechecking for set is done. Added SetType
//  and FunctionType. 
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.7  2000/06/22 17:31:15  cvs
//  Strategy for typechecking of functions was slightly changed. Lists of
//  possible declarations of a function are made for each parameter to the
//  function. Then the first declaration in the intersection of the lists
//  gives the correct function declaration.
//  -- Cindy
//
//  Revision 1.6  2000/04/06 19:14:29  cvs
//  Type checking is done for declaration of variables (simple types), for
//  declaration of new types and for use of basic operators.
//   -- Cindy
//
//  Revision 1.5  2000/02/10 00:37:37  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  1999/11/17 18:31:56  cvs
//  Coded resolution of final type (unfolding overload resolution) and included
//  actions for TCC generation
//
//  Revision 1.3  1999/11/15 22:59:16  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.2  1999/11/15 03:16:30  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.1  1999/11/13 20:01:29  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//
//=============================================================================
package SYM;

import SOM.SOM_ArgumentList;
import SOM.SOM_Declaration;
import SOM.SOM_DeclarationList;
import SOM.SOM_FunctionDeclaration;
import SOM.SOM_FunctionType;
import SOM.SOM_InterfaceItem;
import SOM.SOM_PackageDeclarativeItem;
import SOM.SOM_TreeNode;
import SOM.SOM_TreeNodeList;
import SOM.SOM_TypeDeclaration;
import SOM.SOM_List;

import util.ListEnumeration;

/** This is the <B>SYM_StaticFunctions</B> class
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SYM_StaticFunctions {

	/** This is the static function that does the overloading resolution
	 * @param declList The SOM_DeclarationList that has all the potential
	 * declarations for the node which is being resolved.
	 * @param argList The SOM_ArgumentList which has a list of potential 
	 * matching declarations for each argument.
	 * @return A SOM_TreeList containing the list of all possible matches
	 */
	/** NOTE: Ever since this algorithm has also been used to validate
	 * AssociationListings when doing a lookUp on Name - it also adds to the 
	 * SOM_TreeNodes the declaration that is being processed. That way we can
	 * return both the declaration and its type
	 */
	public static SOM_TreeNodeList buildTree
		(SOM_DeclarationList declList, SOM_ArgumentList argList) {
 
		SOM_TreeNodeList resultTree = new SOM_TreeNodeList();

		int argListCount = argList.countElements();
		int declCount = 0;

		// argList contains node lists. Since overload of variables in
		// same scope are not allowed, these node lists contain only
		// one element. Therefore convert the list of lists to list of nodes.
		SOM_TreeNodeList argNodeList = new SOM_TreeNodeList();
		ListEnumeration argEnum = argList.elements();
		while (argEnum.hasMoreElements()) {
			argNodeList.addElement
				(((SOM_TreeNodeList)argEnum.nextElement()).first());
		}

		ListEnumeration declEnum = declList.elements();	
		SOM_Declaration currDecl, resultDecl;

		SOM_FunctionType fncType;
		SOM_DeclarationList domainList;
		ListEnumeration domainListEnum, argNodeEnum;

		SOM_DeclarationList matchedDeclList = new SOM_DeclarationList();
		ListEnumeration matchedEnum;
		SOM_List listOfLists = new SOM_List();

		SOM_DeclarationList currList;
		SOM_TypeDeclaration currType;
		SOM_TreeNode currNode;

		SOM_DeclarationList resultList = new SOM_DeclarationList();

		while (declEnum.hasMoreElements()) {
			currDecl = (SOM_Declaration)declEnum.nextElement();
			// This is only for functions and design units currently
			if (currDecl instanceof SOM_FunctionDeclaration ||
				currDecl instanceof SOM_PackageDeclarativeItem) {
				// for functions 
				if (currDecl instanceof SOM_FunctionDeclaration) {
					fncType = (SOM_FunctionType)
						((SOM_FunctionDeclaration)currDecl).getType();
					declCount = fncType.getDomainList().size();
				} else {
					// this is for design units
					declCount = ((SOM_PackageDeclarativeItem)currDecl).
						getInterfaceListing().size();
					System.out.println("declCount of designUnit: " 
									   + declCount);
				}

				if (declCount == argListCount) 
					// A match is then possible
					matchedDeclList.addElement(currDecl);
			} else
				return resultTree;
		}

		if (matchedDeclList.isEmpty())
			return resultTree;

		if (matchedDeclList.size() == 1) {
			resultList = matchedDeclList;
			currDecl = (SOM_Declaration)matchedDeclList.getFirst();
			if (currDecl instanceof SOM_PackageDeclarativeItem) {
				ListEnumeration interEnum = 
					((SOM_PackageDeclarativeItem)currDecl).getInterfaceListing().
					interfaces();
						
				domainList = new SOM_DeclarationList();
				SOM_InterfaceItem currItem;
				while (interEnum.hasMoreElements()) {
					currItem = (SOM_InterfaceItem)interEnum.nextElement();
					System.out.println("Add to the domain list: " +
									   currItem.getType().toString());
					domainList.addElement(currItem.getType());
				}
				((SOM_PackageDeclarativeItem)currDecl).setDomainList(domainList);
			}
		} else {
			for (int i = 0; i < argListCount; i++) {
				currList = new SOM_DeclarationList();
				matchedEnum = matchedDeclList.elements();
				while (matchedEnum.hasMoreElements()) {
					currDecl = (SOM_Declaration)matchedEnum.nextElement();
					// for functions retrieve the domain list
					if (currDecl instanceof SOM_FunctionDeclaration) {
						fncType = (SOM_FunctionType)
							((SOM_FunctionDeclaration)currDecl).getType();
						domainList = fncType.getDomainList();
					}
					else {
						// this is for design units
						ListEnumeration interfaceEnum = 
							((SOM_PackageDeclarativeItem)currDecl).getInterfaceListing().
							interfaces();
						
						domainList = new SOM_DeclarationList();
						SOM_InterfaceItem currItem;
						while (interfaceEnum.hasMoreElements()) {
							currItem = (SOM_InterfaceItem)interfaceEnum.
								nextElement();
							System.out.println("Add to the domain list: " +
											   currItem.getType().toString());
							domainList.addElement(currItem.getType());
						}
						((SOM_PackageDeclarativeItem)currDecl).setDomainList(domainList);
					}
					currNode = (SOM_TreeNode)argNodeList.get(i);
					currType = (SOM_TypeDeclaration)domainList.get(i);
					if (SOM_Declaration.isLeftTypeCompatibleWithRightType
						(currType, currNode.getResultType())) {
						if (!SOM_Declaration.isDistinguishable
							(currType, currNode.getResultType())) {
							System.out.println("Exact match is found");
							currList.addFirst(currDecl);
						}
						else
							currList.addElement(currDecl);
					}
				}
				if (currList.isEmpty()) 
					return resultTree;
				
				listOfLists.add(currList);			
			}
			
			if (listOfLists.size() == 1)
				resultList.addFirst
					(((SOM_DeclarationList)listOfLists.first()).first());
			else {
				SOM_DeclarationList firstList = 
					(SOM_DeclarationList)listOfLists.getFirst();
				SOM_DeclarationList otherList;
				ListEnumeration firstEnum, otherEnum;
				SOM_Declaration declOne, declTwo;
				for (int i = 1; i < argListCount; i++) {
					resultList = new SOM_DeclarationList();
					otherList = (SOM_DeclarationList)listOfLists.get(i);
					declOne = (SOM_Declaration) firstList.getFirst();
					declTwo = (SOM_Declaration) otherList.getFirst();
					if (otherList.contains(declOne)) {
						resultList.addFirst(declOne);
						firstList.remove(declOne);
						otherList.remove(declOne);
					} else {
						if (firstList.contains(declTwo)) {
							resultList.addFirst(declTwo);
							firstList.remove(declTwo);
							otherList.remove(declTwo);
						}
					}
					firstEnum = firstList.elements();
					while (firstEnum.hasMoreElements()) {
						declOne = (SOM_Declaration)firstEnum.nextElement();
						if (otherList.contains(declOne)) {
							resultList.add(declOne);
						}
					}
					firstList = resultList;
				}
				
				if (resultList.isEmpty())
					return resultTree;
			} // if listOfLists.size() == 1
		} // if matchedDeclList.size() == 1

		SOM_TreeNode node = new SOM_TreeNode();
		resultDecl = (SOM_Declaration)resultList.getFirst();
		node.setLookUpDecl(resultDecl);
		if (resultList.getFirst() instanceof SOM_FunctionDeclaration) {
			fncType = (SOM_FunctionType)
				((SOM_FunctionDeclaration)resultDecl).getType();
			node.setDomainList(fncType.getDomainList());
			node.setResultType(fncType.getRange());
		} else {
			node.setDomainList(((SOM_PackageDeclarativeItem)resultDecl).getDomainList());
			node.setResultType(resultDecl.getType());
		}
		node.setIsAmbiguous(false);

		argNodeEnum = argNodeList.elements();
		while (argNodeEnum.hasMoreElements()) {
			currNode = (SOM_TreeNode)argNodeEnum.nextElement();
			node.addToChildren(currNode);
		}
		
		resultTree.add(node);
		return resultTree;
	}

	/** To select a matching SOM_TreeNode from a SOM_TreeNodeList. It returns
	 * a unique tree - if one exists.
	 * @param list The SOM_TreeNodeList that is to be searched
	 * @param decl The SOM_TypeDeclaration for which a match is to be found
	 * @return The SOM_TreeNode that fits this decl. If no proper
	 * match is found null is returned
	 */
	
	// This version of selectTreeNode returns a node only if an exact
	// match is found
	
	public static SOM_TreeNode selectTreeNode(SOM_TreeNodeList list,
											  SOM_TypeDeclaration decl) {
		ListEnumeration listEnum = list.elements();
		SOM_TreeNode currNode;

		while (listEnum.hasMoreElements()) {
			currNode = (SOM_TreeNode)listEnum.nextElement();
			if (!SOM_Declaration.isDistinguishable(decl, 
												   currNode.getResultType())) {
				return currNode;
			}
		}
		return null;
	}
	
		/*
	  public static SOM_TreeNodeList buildTree
	  (SOM_DeclarationList declList, SOM_ArgumentList argList)
	  { 
	  SOM_TreeNodeList resultTrees = new SOM_TreeNodeList();
	  SOM_TreeNode tempNode;
	  
	  int argListCount = argList.countElements();
	  
	  ListEnumeration declEnum = declList.elements();	
		SOM_Declaration currDecl;

		SOM_DeclarationList domainList;
		ListEnumeration domainListEnum, argListEnum;

		while (declEnum.hasMoreElements()) {
			currDecl = (SOM_Declaration)declEnum.nextElement();

			// This is only for functions and design units currently
			if (currDecl instanceof SOM_FunctionDeclaration ||
				currDecl instanceof SOM_DesignUnit) {
				// for functions retreive the domain list
				if (currDecl instanceof SOM_FunctionDeclaration) 
					domainList = ((SOM_FunctionDeclaration)currDecl).
						getDomainList();
				else {
					// this is for design units
					ListEnumeration interfaceEnum = 
						((SOM_DesignUnit)currDecl).getInterfaceListing().
						interfaces();

					domainList = new SOM_DeclarationList();
					SOM_InterfaceItem currItem;
					while (interfaceEnum.hasMoreElements()) {
						currItem = (SOM_InterfaceItem)interfaceEnum.
							nextElement();
						domainList.addElement(currItem.getType());
					}
				}
				
				if (domainList.countElements() == argListCount) {
					// A match is then possible. Initialize a node...
					SOM_TreeNode node = new SOM_TreeNode();
					node.setLookUpDecl(currDecl);
					node.setDomainList(domainList);
					node.setResultType(currDecl.getType());
					node.setIsAmbiguous(false);
					
					// now to match each type in the domainList to that in the
					// argument list
					domainListEnum = domainList.elements();
					argListEnum = argList.elements();
					while (domainListEnum.hasMoreElements()) {
						// Note selection of TreeNode must be done on basis of 
						// compatibility of types, rather than exactness of 
						// types...
						tempNode = selectTreeNode
							((SOM_TreeNodeList)argNodeEnum.nextElement(),
							 (SOM_TypeDeclaration)domainListEnum.
							                              nextElement());
						if (tempNode == null) {
							//System.out.println
							//	("tempNode, node " +
							//	 "returned by selectTreeNode is null");
							node.setResultType(null);
							break;
						} else {
							node.addToChildren(tempNode);
							//System.out.println("tempNode (selectTreeNode): "+
							//				   tempNode.getResultType().
							//				   toString());
						}
					} //while (domainListEnum.hasMoreElements())

					if (node.getResultType() != null) {
						SOM_TreeNode selectNode =
							locateAmbiguousTreeNode
							(resultTrees, node.getResultType());
						if (selectNode == null) {
							//System.out.println
							//	("In buildTree, no ambiguous node is present");
							resultTrees.addElement(node);
						}
						else {
							//System.out.println
							//	("In buildTree, ambiguous node found");
							selectNode.setIsAmbiguous(true);
						}
					}
					
				} //if (domainList.countElements() == argListCount)

			}	// if (currDecl instanceof SOM_FunctionDeclaration)		 

		} // while (declEnum.hasMoreElements())

		resultTrees.removeAmbiguousTreeNodes();

		return resultTrees;
	}  

	public static SOM_TreeNode selectTreeNode(SOM_TreeNodeList list,
											  SOM_TypeDeclaration decl) {
		ListEnumeration listEnum = list.elements();
		SOM_TreeNodeList toReturn = new SOM_TreeNodeList();

		SOM_TreeNode currNode;
		//System.out.println("In selectTreeNode, decl is " + decl.toString());
		while (listEnum.hasMoreElements()) {
			currNode = (SOM_TreeNode)listEnum.nextElement();
			// Following lines of code used for debugging
			//System.out.println("currNode is " + 
			//				   currNode.getResultType().toString());
			//System.out.println("isLeftTypeCompatibleWithRightType: " + 
			//				   SOM_Declaration.
			//				   isLeftTypeCompatibleWithRightType
			//				   (decl, currNode.getResultType()));

			if (SOM_Declaration.isLeftTypeCompatibleWithRightType
					(decl, currNode.getResultType())
				) toReturn.addElement(currNode);
		}

		int count = toReturn.countElements();
		//System.out.println("# elements in toReturn : " + count);
		if (count == 0) return null; // no match
		if (count == 1) // unique resolution
			return (SOM_TreeNode)toReturn.first(); 

		// no unique resolution exists. Search for an exact match. If found
		// return that guy, else it is ambiguous
		ListEnumeration toReturnEnum = toReturn.elements();
		SOM_TreeNodeList exactList = new SOM_TreeNodeList();
		while (toReturnEnum.hasMoreElements()) {
			currNode = (SOM_TreeNode)toReturnEnum.nextElement();
			//System.out.println("In selectTreeNode, decl is " +
			//				   decl.toString()); 
			//System.out.println("currNode is " + 
			//				   currNode.getResultType().toString());
			if (!SOM_Declaration.isDistinguishable(decl, 
												   currNode.getResultType())) {
				exactList.addElement(currNode);
				System.out.println("Exact match found");
			}
		}

		if (exactList.countElements() == 1) 
			return (SOM_TreeNode)exactList.first();
		else {
			System.out.println
				("Many matches found in SOM_TreeNode::selectTreeNode");
			return null;
			// Instead of returning null, should I return the first from the
			// original toReturn list and generate a TCC, if one
			// exists??
		}
	}
	*/

	/** To locate an ambigous SOM_TreeNode from a SOM_TreeNodeList.
	 * It returns the first ambiguous node found...
	 * @param list The SOM_TreeNodeList that is to be searched
	 * @param decl The SOM_TypeDeclaration against which an ambguity search
	 * is being made
	 * @return The SOM_TreeNode that is ambiguous. If none is found null
	 * is returned
	 */
	public static SOM_TreeNode locateAmbiguousTreeNode
		(SOM_TreeNodeList list, SOM_TypeDeclaration decl) {
		// what is the definition of ambigious??
		// If the list contains a declaration that is identical to decl then
		// it that declaration is ambiguous
		ListEnumeration listEnum = list.elements();
		SOM_TreeNode treeNode;
		while (listEnum.hasMoreElements()) {
			treeNode = (SOM_TreeNode)listEnum.nextElement();
			if (!SOM_TypeDeclaration.isDistinguishable
				(treeNode.getResultType(),decl))
				return treeNode;
		}
		return null;		
	}
}
