//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: List.java,v 1.6 2000/05/18 18:04:23 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: List.java,v $
//  Revision 1.6  2000/05/18 18:04:23  cvs
//  Due to /projects directory failing and the latest version available is old, I
//  am committing the file again.
//  -- Cindy
//
//  Revision 1.6  2000/05/05 19:36:58  cvs
//  List.java is now inheriting from LinkedList (java1.2).
//  ListEnumeration inherits from ListIterator.
//  This is done so that all predefined methods are available.
//  The methods in List.java and ListEnumeration.java now simply call the
//  corresponding methods from their super class.
//  -- Cindy
//
//  Revision 1.5  2000/03/03 18:38:32  cvs
//  Seriliazing and deserializing the object model - makarand
//
//  Revision 1.4  2000/02/10 03:51:12  cvs
//  Updated the Copyright information in some of the files which were omitted in the previous commit
//  - krishna
//
//  Revision 1.3  1999/11/15 22:59:17  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.2  1999/11/15 03:16:33  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.1  1999/10/27 20:43:59  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:52:50  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.3  1999/07/24 23:21:07  rkamath
//  Added a copy constructor for ListEnumeration and modified concat in List
//  to concatenate a copy of the second argument.....
//
//  Revision 1.2  1999/07/22 22:54:58  rkamath
//  Added a few more methods and totally removed any trace of currency
//
//  Revision 1.1  1999/07/13 19:29:55  rkamath
//  The new util directory with the basic data structures
//
//
//=============================================================================
package util;

import java.lang.Object;
import java.io.*;

import java.util.LinkedList; 

/** This represents the <B>List</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class List
extends LinkedList { 

	/** The primary Constructor
	 * @return Not applicable
	 */
	public List() {
		super();
	}

	/** The Second constructor.
	 * @param data The first object to be stored
	 * return Not applicable
	 */
	public List(Object data) {
		super();
		add(data);
	}
	
	/** To append to the end of the list. Note that the name should have
	 * been more correctly appendElement :-)
	 * @param data The object to be appended to the list
	 * @return void
	 */
	public void addElement(Object data) {
		add(data);
	}

	/** This is to prepend a object to the list at the head
	 * @param data The object to be prepended
	 * @return void
	 */
	public void prependElement(Object data) {
		addFirst(data);
	}

	/** To retrieve a fresh enumeration of the list. 
	 * @return ListEnumeration corresponding to this list
	 */
	public ListEnumeration elements() {
		return new ListEnumeration(listIterator());
	}

	/** To retrieve a fresh reverse enumeration of the list. This
	 * function was necessitated by the need to unfold the overloading
	 * resolution  
	 * @return ListEnumeration corresponding to the reversed list
	 */
	public ListEnumeration reverseElements() {
		ListEnumeration forwardEnum = elements();
		List reverseList = new List();
		Object currData;
		// now reverse this list
		while (forwardEnum.hasMoreElements()) {
			currData = forwardEnum.nextElement();
			reverseList.prependElement(currData);			
		}
		// return the reverse enum
		return reverseList.elements();
	}

	/** To remove a previously added object. If this object was not
	 * added nothing happens. Warning: Currency based operations will be
	 * affected especially if the element being removed is the current
	 * one pointed to by cursor.
	 * @param datIn The object to be removed
	 * @return void
	 */
	public void removeElement(Object dataIn) {
		remove(dataIn);
	}

	/** To Chop off the head of the list. Uses the cannonical pop as
	 * used in Stacks. If the list is empty nothing happens
	 * @return void
	 */
	public void pop() {
		removeFirst();
	}

	/** To concatenate two lists. It concatenates a COPY of the second
	 * argument to the first. Thus only the first argument is modiifed.
	 * @param listOne The first list to which the second will be concatenated
	 * @param listTwo The list to be concatenated to listOne
	 * @return void
	 */
	public static void concat(List listOne, List listTwo) {
		listOne.addAll(listTwo);
	}

	/** To retrieve the element at the head
	 * @return Object at the head of the list
	 */
	public Object first() {
		return getFirst();
	}

	/** To get the count of elements in this list
	 * @return The integer value of size
	 */
	public int countElements() {
		return size();
	}
}

